@@ -1,6 +1,6 @@
 # -*- coding: UTF-8 -*-
 # Copyright 2015-2020 Rumma & Ko Ltd
-# License: BSD (see file COPYING for details)
+# License: GNU Affero General Public License v3 (see file COPYING for details)
 
 """
 A selection of names to be used in tested documents.
@@ -12,7 +12,7 @@ import django
 django.setup()
 from lino.api.shell import *
 from django.utils import translation
-from django.utils.encoding import force_text
+from django.utils.encoding import force_str
 from django.test import Client
 from django.db import connection, reset_queries as reset_sql_queries
 import json
@@ -35,6 +35,8 @@ from lino.sphinxcontrib.actordoc import menuselection_text
 from pprint import pprint
 from lino.utils.diag import visible_for
 from lino.core.utils import full_model_name
+from lino.core.utils import PseudoRequest
+from lino.core.site import html2text
 
 from lino.core.menus import Menu
 from lino.core.actions import ShowTable
@@ -159,43 +161,6 @@ def demo_get(
     #     raise
 
 
-def screenshot(obj, filename, rstname, username='robin'):
-    """Insert a screenshot of the detail window for the given database
-    object.
-
-    Usage example in the source code of
-    http://xl.lino-framework.org/specs/holidays.html.
-
-    Problems: doesn't seem to wait long enough and
-    therefore produces a white .png file.
-
-    How to specify the filename? the current directory when doctest is
-    running is normally the project root, but that's not sure. Best
-    place would be the same directory as the rst file, but how to know
-    that name from within a tested snippet?
-
-    """
-    from lino.api.selenium import Album, runserver
-
-    assert filename.endswith('.png')
-    assert rstname.endswith('.rst')
-
-    ar = rt.login(renderer=dd.plugins.extjs.renderer)
-    ba = obj.get_detail_action()
-    uri = ar.get_detail_url(ba.actor, obj.pk)
-    print(uri)
-
-    def f(driver):
-        app = Album(driver)
-        driver.get("http://127.0.0.1:8000" + uri)
-        # driver.get(uri)
-        app.stabilize()
-        if not driver.get_screenshot_as_file(filename):
-            app.error("Failed to create {0}".format(filename))
-
-    runserver(settings.SETTINGS_MODULE, f)
-
-
 def show_menu_path(spec, language=None):
     """
     Print the menu path of the given actor or action.
@@ -348,7 +313,7 @@ def show_fields_by_type(fldtype):
         for f in model._meta.fields:
             if isinstance(f, fldtype):
                 name = f.name
-                verbose_name = force_text(f.verbose_name).strip()
+                verbose_name = force_str(f.verbose_name).strip()
                 txt = "{verbose_name} ({name})".format(**locals())
                 flds.append(txt)
         if len(flds):
@@ -516,3 +481,104 @@ def pprint_json_string(s):
                      indent=2,
                      sort_keys=True,
                      separators=(",", ": ")))
+
+
+def show_dashboard(username, **options):
+    """Show the dashboard of the given user.
+
+    Useful options:
+
+    - ignore_links=True
+
+    For more options, see
+    https://pypi.org/project/html2text/ and
+    https://github.com/Alir3z4/html2text/blob/master/docs/usage.md
+
+    Note that this is currently not much used because the result is difficult to
+    maintain.  One reason for this is that :func:`naturaltime` (from
+    :mod:`django.contrib.humanize.templatetags.humanize`) ignores demo_date and
+    therefore produces results that depend on the current date/time.
+
+
+
+    """
+    request = PseudoRequest(username)
+    ui = settings.SITE.kernel.text_renderer.front_end
+    html = settings.SITE.get_main_html(request, extjs=ui)
+    print(html2text(html, **options))
+
+def menu2rst(mnu, level=1):
+    """Recursive utility used by :func:`show_menu`.
+    """
+    if not isinstance(mnu, Menu):
+        return str(mnu.label)
+
+    has_submenus = False
+    for i in mnu.items:
+        if isinstance(i, Menu):
+            has_submenus = True
+    items = [menu2rst(mi, level + 1) for mi in mnu.items]
+    if has_submenus:
+        s = rstgen.ul(items).strip() + '\n'
+        if mnu.label is not None:
+            s = str(mnu.label) + ' :\n\n' + s
+    else:
+        s = ', '.join(items)
+        if mnu.label is not None:
+            s = str(mnu.label) + ' : ' + s
+    return s
+
+
+def show_menu(username, language=None, stripped=True, level=1):
+    """
+
+    Render the main menu for the given user as a reStructuredText formatted
+    bullet list.
+
+    :language: explicitly select another language than that
+               specified in the requesting user's :attr:`language
+               <lino.modlib.users.models.User.language>` field.
+    :stripped: remove lots of blanklines which are necessary for
+               reStructuredText but disturbing in a doctest
+               snippet.
+
+    """
+    ar = rt.login(username)
+    user = ar.get_user()
+    if language is None:
+        language = user.language
+    with translation.override(language):
+        mnu = settings.SITE.get_site_menu(user.user_type)
+        s = menu2rst(mnu, level)
+        if stripped:
+            for ln in s.splitlines():
+                if ln.strip():
+                    print(ln)
+        else:
+            print(s)
+
+
+def walk_store_fields(only_detail_fields=False):
+    """
+
+    Iterate over all the atomizers (:class:`lino.core.store.StoreField`) in this
+    application.
+
+    """
+    # found = set()
+    for a in actors.actors_list:
+        if not a.is_abstract():
+            if a.get_handle_name is not None:
+                continue  # debts.PrintEntriesByBudget
+            ah = a.get_handle()
+            if not hasattr(ah, 'store'):
+                raise Exception("{} has no store".format(ah.actor))
+            if only_detail_fields:
+                lst = ah.store.detail_fields
+            else:
+                lst = ah.store.all_fields
+            for sf in lst:
+                # if sf.field in found:
+                #     continue
+                # found.add(sf.field)
+                yield a, sf
