@@ -1,6 +1,6 @@
 # -*- coding: UTF-8 -*-
 # Copyright 2009-2021 Rumma & Ko Ltd
-# License: BSD (see file COPYING for details)
+# License: GNU Affero General Public License v3 (see file COPYING for details)
 
 """This defines the :class:`Action` class and the :func:`action`
 decorator, and some of the standard actions.  See :ref:`dev.actions`.
@@ -9,10 +9,10 @@ decorator, and some of the standard actions.  See :ref:`dev.actions`.
 
 import logging; logger = logging.getLogger(__name__)
 
-from django.utils.translation import ugettext_lazy as _
-from django.utils.translation import ugettext as gettext
+from django.utils.translation import gettext_lazy as _
+from django.utils.translation import gettext
 from django.utils.text import format_lazy
-from django.utils.encoding import force_text
+from django.utils.encoding import force_str
 from django.conf import settings
 from django.db import models
 
@@ -47,7 +47,7 @@ def discover_choosers():
 def install_layout(cls, k, layout_class, **options):
     """
     - `cls` is the actor (a class object)
-    - `k` is one of 'detail_layout', 'insert_layout', 'params_layout', 'card_layout'
+    - `k` is one of 'detail_layout', 'insert_layout', 'params_layout', 'card_layout', 'item_layout'
     - `layout_class`
 
     """
@@ -148,21 +148,11 @@ class Action(Parametrizable, Permittable):
     _layout_class = layouts.ActionParamsLayout
 
     label = None
-    """
-    The label of this action. A short descriptive text in user
-    language. Used e.g. on menu items. Also on toolbar buttons if they
-    have neither :attr:`icon_name` nor :attr:`button_text`.
-    """
-
     button_text = None
-    """
-    The text to appear on buttons for this action. If this is not
-    defined, the :attr:`label` is used.
-    """
 
     button_color = None
     """
-    The color to be used on icon-less buttons for this action
+    The olor to be used on icon-less buttons for this action
     (i.e. which have no :attr:`icon_name`).  See also
     :attr:`lino.core.site.Site.use_silk_icons`.
 
@@ -259,15 +249,6 @@ class Action(Parametrizable, Permittable):
 
     """
     help_text = None
-    """
-    A help text that shortly explains what this action does.  In a
-    graphical user interface this will be rendered as a **tooltip**
-    text.
-
-    If this is not given by the code, Lino will potentially set it at
-    startup when loading the :xfile:`help_texts.py` files.
-
-    """
 
     submit_form_data = False
     """
@@ -352,16 +333,18 @@ class Action(Parametrizable, Permittable):
 
     opens_a_window = False
 
-    hide_top_toolbar = False
+    hide_top_toolbar = False  # 20210509
     """
-    Used internally if :attr:`opens_a_window` to say whether the
-    window has a top toolbar.
+    Move toolbar to bottom on the window opened by this action.
+
+    This is used by ExtJS for ShowInsert, but has no effect in React.
+
+    TODO: rename this to something like ``toolbar_location = "bottom"``.
 
     """
-    hide_navigator = False
+    hide_navigator = False  # 20210509
     """
-    Used internally if :attr:`opens_a_window` to say whether the
-    window has a navigator.
+    Hide navigator actions on window opened by this action.
 
     """
 
@@ -373,15 +356,19 @@ class Action(Parametrizable, Permittable):
 
     show_in_bbar = True
     """
-    Whether this action should be displayed as a button in the toolbar
-    and the context menu of a full grid.
+
+    Whether this action should be displayed in the toolbar.
+
+    TODO: rename this to `show_in_toolbar`.
+
+    In ExtJS this will also cause it to be in the context menu of a grid.
 
     For example the :class:`CheckinVisitor
-    <lino_xl.lib.reception.models.CheckinVisitor>`,
+    <lino_xl.lib.reception.CheckinVisitor>`,
     :class:`ReceiveVisitor
-    <lino_xl.lib.reception.models.ReceiveVisitor>` and
+    <lino_xl.lib.reception.ReceiveVisitor>` and
     :class:`CheckoutVisitor
-    <lino_xl.lib.reception.models.CheckoutVisitor>` actions have this
+    <lino_xl.lib.reception.CheckoutVisitor>` actions have this
     attribute explicitly set to `False` because otherwise they would be
     visible in the toolbar.
     """
@@ -627,7 +614,7 @@ class Action(Parametrizable, Permittable):
             name)
 
     def __str__(self):
-        # return force_text(self.label)
+        # return force_str(self.label)
         # return str(self.get_label())
         return str(self.get_label())
 
@@ -829,10 +816,9 @@ class ShowEmptyTable(ShowDetail):
     use_param_panel = True
     action_name = 'show'
     default_format = 'html'
-    # ~ hide_top_toolbar = True
-    hide_navigator = True
     icon_name = None
     callable_from = 't'
+    hide_navigator = True  # 20210509
 
     # def attach_to_actor(self, actor, name):
     #     self.label = actor.label
@@ -864,9 +850,9 @@ class ShowInsert(TableAction):
     show_in_workflow = False
     opens_a_window = True
     window_type = 'i'
-    hide_navigator = True
+    hide_navigator = True  # 20210509
+    hide_top_toolbar = True  # 20210509
     sort_index = 10
-    hide_top_toolbar = True
     # required_roles = set([SiteUser])
     action_name = 'insert'
     key = keyboard.INSERT  # (ctrl=True)
@@ -882,9 +868,9 @@ class ShowInsert(TableAction):
         return super(ShowInsert, self).attach_to_actor(owner, name)
 
     def get_action_title(self, ar):
-        # return _("Insert into %s") % force_text(ar.get_title())
+        # return _("Insert into %s") % force_str(ar.get_title())
         if ar.actor.model is None:
-            return _("Insert into %s") % force_text(ar.get_title())
+            return _("Insert into %s") % force_str(ar.get_title())
         return format_lazy(_("New {}"), ar.actor.model._meta.verbose_name)
 
     def get_window_layout(self, actor):
@@ -996,8 +982,9 @@ class SubmitDetail(SaveGridCell):
     callable_from = 'd'
 
     def run_from_ui(self, ar, **kw):
-        # logger.info("20140423 SubmitDetail")
+        # logger.info("20210213a SubmitDetail")
         for elem in ar.selected_rows:
+            # logger.info("20210213b SubmitDetail %s", elem)
             elem.save_existing_instance(ar)
             if not settings.SITE.is_installed("react"):
                 # No point in clos
@@ -1035,10 +1022,6 @@ class CreateRow(Action):
             # file upload
             ar.set_response(rows=[ar.ah.store.row2list(ar, elem)])
             ar.set_response(navinfo=navinfo(ar.data_iterator, elem))
-        else:
-            # Must set text/html for file uploads, otherwise the
-            # browser adds a <PRE></PRE> tag around the AJAX response.
-            ar.set_content_type('text/html')
 
         # if ar.actor.stay_in_grid and ar.requesting_panel:
         if ar.actor.stay_in_grid:
@@ -1092,6 +1075,13 @@ class SubmitInsert(CreateRow):
         # is going to be closed (this disturbs at least in ticket
         # #219)
         ar.requesting_panel = None
+
+        if ar.actor.handle_uploaded_files is not None:
+            # Must set text/html for file uploads, otherwise the
+            # browser adds a <PRE></PRE> tag around the AJAX response.
+            # 20210217 And this is true also in case of a ValidationError
+            ar.set_content_type('text/html')
+
         # print("20201230 SubmitInsert.run_from_ui", ar)
         if ar.actor.handle_uploaded_files is not None and len(ar.request.FILES.getlist("file")) > 1:
             # Multiple uploads possible, note plural method names.
