@@ -1,6 +1,6 @@
 # -*- coding: UTF-8 -*-
 # Copyright 2009-2021 Rumma & Ko Ltd
-# License: BSD (see file COPYING for details)
+# License: GNU Affero General Public License v3 (see file COPYING for details)
 
 """This defines :class:`Actor` and related classes.
 
@@ -18,7 +18,7 @@ from types import GeneratorType
 
 from django.db import models
 from django.conf import settings
-from django.utils.translation import ugettext_lazy as _
+from django.utils.translation import gettext_lazy as _
 
 from lino.utils import isiterable
 from lino.core import fields
@@ -135,6 +135,13 @@ class ActorMetaClass(type):
         declared_known_values = classDict.pop('known_values', None)
         if declared_known_values is not None:
             classDict.update(_known_values=declared_known_values)
+#         if 'editable' in classDict:
+#             raise ChangedAPI("""
+# 20210512 {} : replace 'editable = False' by "
+# @classmethod
+# def hide_editing(cls, user_type):
+#     return True
+# """.format(classname))
         declared_editable = classDict.pop('editable', None)
         if declared_editable is not None:
             classDict.update(_editable=declared_editable)
@@ -234,6 +241,7 @@ class ActorMetaClass(type):
 
     @property
     def editable(cls):
+        # See docs/dev/disable.rst
         return cls.get_actor_editable()
 
 
@@ -245,13 +253,6 @@ class Actor(with_metaclass(ActorMetaClass, type('NewBase', (actions.Parametrizab
     <lino.core.choicelists.ChoiceList>` and :class:`Frame
     <lino.core.frames.Frame>`.
 
-    .. attribute:: label
-
-        The text to appear e.g. on a button that will call the default
-        action of an actor.  This attribute is *not* inherited to
-        subclasses.  For :class:`Actor` subclasses that don't have a
-        label, Lino will call :meth:`get_actor_label`.
-
     .. attribute:: known_values
 
         A `dict` of `fieldname` -> `value` pairs that specify "known values".
@@ -278,23 +279,6 @@ class Actor(with_metaclass(ActorMetaClass, type('NewBase', (actions.Parametrizab
         implement dynamic columns depending on it's master_instance.
 
 
-    .. classmethod:: get_row_classes(self, ar)
-
-        If a method of this name is defined on an actor, then it must
-        be a class method which takes an :class:`ar
-        <lino.core.requests.BaseRequest>` as single argument and
-        returns either None or a string "red", "green" or "blue"
-        (todo: add more colors and styles). Example::
-
-            @classmethod
-            def get_row_classes(cls,obj,ar):
-                if obj.client_state == ClientStates.newcomer:
-                    return 'green'
-
-        Defining this method will cause an additional special
-        `RowClassStoreField` field in the :class:`lino.core.Store`
-        objects of this actor.
-
     .. classmethod:: get_welcome_messages(self, ar)
 
         If a method of this name is defined on an actor, then it must
@@ -307,16 +291,18 @@ class Actor(with_metaclass(ActorMetaClass, type('NewBase', (actions.Parametrizab
         Note that this handler will be called independently of whether
         the user has permission to view the actor or not.
     """
+    _detail_action_class = actions.ShowDetail
+
     required_roles = set([SiteUser])
-    """See :attr:`lino.core.permissions.Permittable.required_roles`"""
+    """See :doc:`/dev/perms`."""
 
     model = None
     """The model on which this table iterates.
 
-    The application programmer can specify either the model class
-    itself or a string of type ``'app.Model'``.
+    The :term:`application developer` can specify either the model class itself
+    or a string of type ``'app.Model'``.
 
-    This should be `None` on all tables which are not subclass of
+    This should be `None` on all tables that are not subclass of
     :class:`lino.core.dbtables.Table`.
 
     """
@@ -374,16 +360,9 @@ class Actor(with_metaclass(ActorMetaClass, type('NewBase', (actions.Parametrizab
     """
 
     details_of_master_template = _("%(details)s of %(master)s")
-    """Used to build the title of a request on this table when it is a
-    slave table (i.e. :attr:`master` is not None). The default value
-    is defined as follows::
-
-        details_of_master_template = _("%(details)s of %(master)s")
-
-    """
 
     parameters = None
-    "See :attr:`lino.core.utils.Parametrizable.parameters`."
+    # See :attr:`lino.core.utils.Parametrizable.parameters`.
 
     _layout_class = layouts.ParamsLayout
     _state_to_disabled_actions = None
@@ -395,7 +374,7 @@ class Actor(with_metaclass(ActorMetaClass, type('NewBase', (actions.Parametrizab
     Set this to `True` on a workflow if you want to disable workflow
     control based on the state of the object.
 
-    Note that you must set this to True before importing any library workflows
+    Note that you must set this to True *before* importing any library workflows
     because permission handlers are defined when a workflow is imported. """
 
     sort_index = 60
@@ -412,28 +391,8 @@ class Actor(with_metaclass(ActorMetaClass, type('NewBase', (actions.Parametrizab
     """
 
     simple_parameters = None
-    """A tuple of names of filter parameters which are handled
-    automatically.
-
-    Application developers should not set this attribute directly,
-    they should rather define a :meth:`get_simple_parameters` on the
-    model.
-
-    """
 
     hidden_elements = frozenset()
-    """A set of names of layout elements which are hidden by default.
-
-    The default is an empty set except for
-    :class:`lino.core.dbtables.Table` where this will be populated from
-    :attr:`hidden_elements <lino.core.model.Model.hidden_elements>`
-    of the :class:`lino.core.model.Model`.
-
-    Note that these names are not being verified to be names of
-    existing fields. This fact is being used by UNION tables like
-    :class:`lino_xl.lib.vat.IntracomInvoices`
-
-    """
 
     detail_html_template = 'bootstrap3/detail.html'
     """The template to be used for rendering a row of this actor as a
@@ -449,48 +408,11 @@ class Actor(with_metaclass(ActorMetaClass, type('NewBase', (actions.Parametrizab
     get_welcome_messages = None
     get_row_classes = None
     window_size = None
-    """Set this to a tuple of `(height, width)` to have this actor
-    display in a modal non-maximized window.
-
-    - `height` must be either an integer expressing a number of rows
-      or the string "auto".  If it is auto, then the window should not
-      contain any v-flexible component.
-
-    - `width` must be either an integer expressing a number of rows
-      or a string of style "90%".
-
-      Note that a relative width will be converted to a number of
-      pixels when the window is rendered for the first time. That is,
-      if you close the window, resize your browser window and reopen
-      the same window, you will get the old size.
-
-    """
 
     default_list_action_name = 'grid'
-    default_elem_action_name = 'detail'
+    # default_elem_action_name = 'detail'
 
-    # update_required = set()
-    # delete_required = set()
-
-    editable = None
-    """Set this explicitly to `True` or `False` to make the whole table
-    editable or not.  Otherwise Lino will guess what you want during
-    startup and set it to `False` if the actor is a Table and has a
-    `get_data_rows` method (which usually means that it is a virtual
-    table), otherwise to `True`.
-
-    Non-editable actors won't even call :meth:`get_view_permission`
-    for actions whose :attr:`readonly
-    <lino.core.actions.Action.readonly>` is `False`.
-
-    The
-    :class:`changes.Changes <lino.modlib.changes.Changes>`
-    table is an example where this is being used: nobody should
-    ever edit something in the table of Changes.  The user
-    interface uses this to generate optimized JS code for this
-    case.
-
-    """
+    # editable = None
 
     auto_apply_params = True
     """Whether the parameter values of the parameter panel should be
@@ -499,157 +421,45 @@ class Actor(with_metaclass(ActorMetaClass, type('NewBase', (actions.Parametrizab
     """
 
     insert_layout_width = 60
-    """
-    When specifying an :attr:`insert_layout` using a simple a multline
-    string, then Lino will instantiate a FormPanel with this width.
-    """
-
     hide_window_title = False
-    """
-    This is set to `True` e.h. in home pages
-    (e.g. :class:`lino_welfare.modlib.pcsw.models.Home`).
-
-    """
-
     allow_create = True
-    """
-    If this is False, the table won't have any insert_action.
-    """
-
+    allow_delete = True
     hide_headers = False
-    """Set this to True in order to hide the column headers.
 
-    This is ignored when the table is rendered in an ExtJS grid.
+    hide_top_toolbar = False  # 20210509
+    """
+    Have the toolbar at the bottom of the window instead of the top.
+
+    See :ref:`dev.actor_config.hide_top_toolbar`.
+
+    Deprecated. Supported only in :term:`ExtJS front end`.
     """
 
-    hide_top_toolbar = False
+    hide_navigator = False  # 20210509
     """
-    Whether a Detail Window should have navigation buttons, a "New"
-    and a "Delete" buttons.  In ExtJS UI also influences the title of
-    a Detail Window to specify only the current element without
-    prefixing the Tables's title.
+    Whether to hide navigation functionality.
 
-    If used in a grid view in React will remove the top toolbar
-    and selection tools.
-
-    This option is `True` in
-    :class:`lino.models.SiteConfigs`,
-    :class:`lino_welfare.pcsw.models.Home`,
-    :class:`lino.modlib.users.desktop.MySettings`,
-    :class:`lino_xl.cal.CalenderView`.
+    See :ref:`dev.actor_config.hide_navigator`.
 
     """
 
     simple_slavegrid_header = False
-    """
-    If True the slave grid in a detail will be simplified
-    """
-
-    # paginator_rowsPerPageOptions = None
-    # """Array of integer values to display inside rows per page dropdown in the paginator."""
 
     paginator_template = None
-    """
-    Paginator elements can be customized using the template property using the predefined keys, default value is
-    "FirstPageLink PrevPageLink PageLinks NextPageLink LastPageLink RowsPerPageDropdown".
-    Here are the available elements that can be placed inside a paginator.
-
-    FirstPageLink
-    PrevPageLink
-    PageLinks
-    NextPageLink
-    LastPageLink
-    RowsPerPageDropdown
-    CurrentPageReport
-    """
 
     _label = None
     _editable = None
     _known_values = {}
-
     title = None
-    """
-    The text to appear e.g. as window title when the actor's default
-    action has been called.  If this is not set, Lino will use the
-    :attr:`label` as title.
-    """
-
     button_text = None
-    """The text to appear on buttons of a ShowSlaveTable action for this
-    actor.
-
-    """
-
     label = None
-
     default_action = None
     actor_id = None
 
     detail_layout = None
-    '''
-    Define the layout to use for the detail window.  Actors with
-    :attr:`detail_layout` will get a `show_detail` action.
-
-    When you define a :attr:`detail_layout`, you will probably also
-    want to define a :attr:`insert_layout`.
-
-    The :attr:`detail_layout` is normally an instance of
-    :class:`DetailLayout <lino.core.layouts.DetailLayout>` or a
-    subclass thereof.  For example::
-
-        class FooDetail(dd.DetailLayout):
-            ...
-
-        class Foos(dd.Table):
-            ...
-            detail_layout = FooDetail()
-
-    It is possible and recommended to specify :attr:`detail_layout` as
-    a string, in which case it will be resolved at startup as follows:
-
-    If the string contains at least one newline (or no newline and
-    also no dot) then it is taken as the :attr:`main` of a
-    :class:`DetailLayout <lino.core.layouts.DetailLayout>`.
-    For example::
-
-        class Foos(dd.Table):
-            ...
-            detail_layout = """
-            id name
-            description
-            """
-
-    If the string contains a dot ('.') and *does not contain* any
-    newlines, then Lino takes this as the name of the class to be
-    instantiated and used.
-
-    For example::
-
-        class Courses(dd.Table):
-            ...
-            detail_layout = 'courses.CourseDetail'
-
-    This feature makes it possible to override the detail layout in an
-    extended plugin. Before this you had to define a new class and to
-    assign an instance of that class to every actor which uses it.
-    But e.g. in :mod:`lino_xl.lib.courses` we have a lot of subclasses
-    of the :class:`Courses` actor.
-    '''
-
     insert_layout = None
-    """
-    Define the form layout to use for the insert window.
-
-    If there's a :attr:`detail_layout` but no :attr:`insert_layout`,
-    the table won't have any (+) button to create a new row via a
-    dialog window, but users can still create rows by writing into the
-    phantom row. Example of this is
-    :class:`lino_xl.lib.courses.Topics` which has a detail layout
-    with slave tables, but the model itself has only two fields (id
-    and name) and it makes no sense to have an insert window.
-    """
-
     card_layout = None
+    item_layout = None
     """
     Define a layout for a card view of the table."""
 
@@ -657,15 +467,6 @@ class Actor(with_metaclass(ActorMetaClass, type('NewBase', (actions.Parametrizab
     insert_template = None    # deprecated: use detail_layout instead
 
     help_text = None
-    """
-    A help text that shortly explains what the default action of this
-    actor does.  In a graphical user interface this will be rendered
-    as a **tooltip** text.
-
-    If this is not given by the code, Lino will potentially set it at
-    startup when loading the :xfile:`help_texts.py` files.
-    """
-
     detail_action = None
     update_action = None
     insert_action = None
@@ -684,16 +485,8 @@ class Actor(with_metaclass(ActorMetaClass, type('NewBase', (actions.Parametrizab
 
     """
     sum_text_column = 0
-    """The index of the column which should hold the text to display on
-    the totals row (returned by :meth:`get_sum_text`).
-
-    """
 
     preview_limit = None
-    """For non-table actors this is always `None`, otherwise see
-    :attr:`lino.core.tables.AbstractTable.preview_limit`.
-
-    """
 
     handle_uploaded_files = None
     """
@@ -710,7 +503,7 @@ class Actor(with_metaclass(ActorMetaClass, type('NewBase', (actions.Parametrizab
         Actor-level hook for overriding the formating when rendering
         this table as plain html.
 
-        For example :class:`ml.cal.Events` overrides this.
+        For example :class:`lino_xl.lib.cal.Events` overrides this.
         """
         pass
 
@@ -723,8 +516,10 @@ class Actor(with_metaclass(ActorMetaClass, type('NewBase', (actions.Parametrizab
         return settings.SITE.is_installed(self.app_label)
 
     @classmethod
-    def get_widget_options(self, name, **options):
-        return options
+    def get_widget_options(cls, name, **options):
+        if cls.model is None:
+            return options
+        return cls.model.get_widget_options(name, **options)
 
     @classmethod
     def get_chooser_for_field(cls, fieldname):
@@ -812,10 +607,8 @@ class Actor(with_metaclass(ActorMetaClass, type('NewBase', (actions.Parametrizab
     @classmethod
     def make_disabled_fields(cls, obj, ar):
         """
-        Return a set of disabled fields for the specified object and
-        request.
-
-        See :doc:`/dev/disabled_fields`.
+        Used internally. Return a set of disabled fields for the specified
+        object and request. See :doc:`/dev/disable`.
         """
 
         s = set()
@@ -843,10 +636,10 @@ class Actor(with_metaclass(ActorMetaClass, type('NewBase', (actions.Parametrizab
         The default implementation assumes that you navigate on the
         :attr:`data_iterator`.
 
-        :class:`lino_xl.lib.cal.CalendarView` overrides this.
+        :class:`lino_xl.lib.calview.DayNavigator` overrides this.
 
         """
-        return navinfo(ar.data_iterator, obj)
+        return navinfo(ar.data_iterator, obj, ar.limit)
 
     @classmethod
     def is_valid_row(self, row):
@@ -883,14 +676,6 @@ class Actor(with_metaclass(ActorMetaClass, type('NewBase', (actions.Parametrizab
 
     @classmethod
     def summary_row(cls, ar, obj, **kw):
-        """
-        Return a HTML representation of the given data row `obj` for usage in a
-        summary panel.
-
-        The default implementation calls
-        :meth:`lino.core.model.Model.summary_row`.
-
-        """
         return obj.summary_row(ar, **kw)
 
     @classmethod
@@ -993,6 +778,7 @@ class Actor(with_metaclass(ActorMetaClass, type('NewBase', (actions.Parametrizab
         actions.install_layout(cls, 'insert_layout', layouts.InsertLayout,
             window_size=(cls.insert_layout_width, 'auto'))
         actions.install_layout(cls, 'card_layout', layouts.DetailLayout)
+        actions.install_layout(cls, 'item_layout', layouts.DetailLayout)
 
         if cls.abstract:
             return
@@ -1063,8 +849,29 @@ class Actor(with_metaclass(ActorMetaClass, type('NewBase', (actions.Parametrizab
         return cls._known_values
 
     @classmethod
-    def get_actor_editable(self):
-        return self._editable
+    def hide_editing(cls, user_type):
+        """
+
+        Whether users of the given :term:`user type` can edit in this
+        :term:`data window`.
+
+        Returns `False` by default: a user who has view permission also has
+        permission to edit the data they see.
+
+        If this returns `True`, then Lino won't even call :meth:`disable_delete`
+        or :meth:`disabled_fields`.
+
+        This is similar to :attr:`editable`, but it allows you to keep editing
+        functionality for certain user types.
+
+        Usage examples see :ref:`dev.actor_config.editing`.
+
+        """
+        return not cls.editable
+
+    @classmethod
+    def get_actor_editable(cls):
+        return cls._editable
 
     @classmethod
     def hide_elements(self, *names):
@@ -1088,10 +895,6 @@ class Actor(with_metaclass(ActorMetaClass, type('NewBase', (actions.Parametrizab
 
     @classmethod
     def get_create_permission(self, ar):
-        """Dynamic test per request.  This is being called only when
-        :attr:`allow_create` is True.
-
-        """
         if not settings.SITE.user_types_module:
             return True
         if ar.get_user().user_type.readonly:
@@ -1100,15 +903,60 @@ class Actor(with_metaclass(ActorMetaClass, type('NewBase', (actions.Parametrizab
 
     @classmethod
     def get_row_permission(cls, obj, ar, state, ba):
-        """Returns True or False whether the given action request
-        ActionRequest `ar` is allowed on the given row instance `row`.
-
         """
+        Whether to allow the given action request `ar` on the given
+        :term:`database row` `row`.
+        """
+
         if ba.action.readonly:
             return True
-        if ar.get_user().user_type.readonly:
+        user_type = ar.get_user().user_type
+        if user_type.readonly:
             return False
-        return cls.editable
+        return not cls.hide_editing(user_type)
+
+    @classmethod
+    def get_request_detail_action(cls, ar):
+        """
+        Return the detail action of this actor, respecting the user's view
+        permissions.
+
+        This is equivalent to the actors's :attr:`detail_action`, except when
+        the model's :meth:`get_detail_action` method returns a table for which
+        the user has no view permission.  In that case we don't want Lino to
+        give up too quickly because there is still a possibility that the same
+        layout is used on other tables, to which the user does have permission.
+
+        An example is described in :ref:`avanti.specs.get_request_detail_action`
+
+        """
+        ut = ar.get_user().user_type
+        ba = cls.detail_action
+        if ar.actor and ba.action is ar.actor.detail_action.action:
+            # 20210223 When this actor uses the same action, don't return
+            # the default area actor because the user might not have
+            # permission to see the defining actor. See
+            # specs/avanti/courses.rst
+            # return ar.actor.detail_action
+            ba = ar.actor.detail_action
+            if ba.allow_view(ut):
+            # if ba.get_view_permission(ut):
+                # print("20210223", ba, "is allowed for", ut)
+                return ba
+        # e.g. the owner of a cal.Event
+        # wl = table.get_detail_layout()
+        # for ds in wl.layout.get_datasources():
+        for ds in ba.action.owner.get_datasources():
+            if ds.default_action is None:
+                # print("20210224", ds)
+                continue
+            if ds.default_action.get_view_permission(ut):
+                ba = ds.detail_action
+                # if ba.allow_view(ut):
+                if ba.get_view_permission(ut):
+                    # print("20210223", ba, "is allowed for", ut)
+                    return ba
+        return None
 
     @classmethod
     def _collect_actions(cls):
@@ -1119,23 +967,11 @@ class Actor(with_metaclass(ActorMetaClass, type('NewBase', (actions.Parametrizab
         Also fill :attr:`_actions_list`.
         """
 
-        default_action = cls.get_default_action()
-
-        if default_action is not None:
-            # cls.default_action = cls._bind_action(default_action)
-            cls.default_action = cls._bind_action(
-                'default_action', default_action, True)
-
         if cls.detail_layout:
-            if default_action and isinstance(
-                    default_action, actions.ShowDetail) and \
-                    default_action.owner == cls.detail_layout:
-                dtla = default_action
+            if cls.detail_action and cls.detail_action.action.owner == cls.detail_layout:
+                dtla = cls.detail_action.action
             else:
-                if cls.detail_action and cls.detail_action.action.owner == cls.detail_layout:
-                    dtla = cls.detail_action.action
-                else:
-                    dtla = actions.ShowDetail(cls.detail_layout)
+                dtla = cls._detail_action_class(cls.detail_layout)
             cls.detail_action = cls._bind_action('detail_action', dtla, True)
             if cls.use_detail_param_panel:
                 cls.detail_action.action.use_param_panel = True
@@ -1145,7 +981,6 @@ class Actor(with_metaclass(ActorMetaClass, type('NewBase', (actions.Parametrizab
                 cls.submit_detail = cls._bind_action(
                     'submit_detail', SUBMIT_DETAIL, True)
 
-
         # avoid inheriting the following actions from parent:
         cls.insert_action = None
         cls.delete_action = None
@@ -1160,7 +995,7 @@ class Actor(with_metaclass(ActorMetaClass, type('NewBase', (actions.Parametrizab
                 if cls.insert_layout:
                     cls.insert_action = cls._bind_action(
                         'insert_action', cls.get_insert_action(), True)
-            if not cls.hide_top_toolbar:
+            if cls.allow_delete:
                 cls.delete_action = cls._bind_action(
                     'delete_action', DELETE_ACTION, True)
             cls.update_action = cls._bind_action(
@@ -1170,7 +1005,6 @@ class Actor(with_metaclass(ActorMetaClass, type('NewBase', (actions.Parametrizab
                     'validate_form', VALIDATE_FORM, True)
 
         if is_string(cls.workflow_owner_field):
-        # if isinstance(cls.workflow_owner_field, string_types):
             cls.workflow_owner_field = cls.get_data_elem(
                 cls.workflow_owner_field)
         if is_string(cls.workflow_state_field):
@@ -1182,7 +1016,6 @@ class Actor(with_metaclass(ActorMetaClass, type('NewBase', (actions.Parametrizab
                         cls,cls.workflow_state_field))
             cls.workflow_state_field = fld
 
-
         # note that the fld may be None e.g. cal.Component
         if cls.workflow_state_field is not None:
             for a in cls.workflow_state_field.choicelist.workflow_actions:
@@ -1200,6 +1033,13 @@ class Actor(with_metaclass(ActorMetaClass, type('NewBase', (actions.Parametrizab
                 if isinstance(v, actions.Action):
                     cls._bind_action(k, v, False)
 
+        da = cls.get_default_action()
+        if da is not None:
+            if isinstance(da, actions.Action):
+                cls.default_action = cls._bind_action('default_action', da, True)
+            else:
+                cls.default_action = da
+
         cls._actions_list.sort(
             key=lambda a: (a.action.sort_index, a.action.action_name))
         # cls._actions_list = tuple(cls._actions_list)
@@ -1297,9 +1137,6 @@ class Actor(with_metaclass(ActorMetaClass, type('NewBase', (actions.Parametrizab
 
     @classmethod
     def get_actor_label(self):
-        """Compute the label of this actor.
-
-        """
         return self._label or self.__name__
 
     @classmethod
@@ -1332,17 +1169,6 @@ class Actor(with_metaclass(ActorMetaClass, type('NewBase', (actions.Parametrizab
 
     @classmethod
     def get_title(self, ar):
-        """Return the title of this actor for the given action request `ar`.
-
-        The default implementation calls :meth:`get_title_base` and
-        :meth:`get_title_tags` and returns a string of type `BASE [
-        (TAG, TAG...)]`.
-
-        Override this if your table's title should mention for example
-        filter conditions.  See also :meth:`Table.get_title
-        <lino.core.dbtables.Table.get_title>`.
-
-        """
         title = self.get_title_base(ar)
         # tags = list(self.get_title_tags(ar))
         tags = [str(t) for t in self.get_title_tags(ar)]
@@ -1352,14 +1178,6 @@ class Actor(with_metaclass(ActorMetaClass, type('NewBase', (actions.Parametrizab
 
     @classmethod
     def get_title_base(self, ar):
-        """
-        Return the base part of the title. This should be a translatable
-        string. This is called by :meth:`get_title` to construct the
-        actual title.
-
-        It is also called by
-        :meth:`lino.core.dashboard.DashboardItem.render_request`
-        """
         title = self.title or self.label
         # if self.master is not None:
         if ar.master_instance is not None:
@@ -1370,11 +1188,6 @@ class Actor(with_metaclass(ActorMetaClass, type('NewBase', (actions.Parametrizab
 
     @classmethod
     def get_title_tags(self, ar):
-        """
-        Yield a list of translatable strings to be added to the base part
-        of the title. This is called by :meth:`get_title` to construct
-        the actual title.
-        """
         if isinstance(self.parameters, ParameterPanel):
             for t in self.parameters.get_title_tags(ar):
                 yield t
@@ -1419,10 +1232,6 @@ class Actor(with_metaclass(ActorMetaClass, type('NewBase', (actions.Parametrizab
 
     @classmethod
     def get_simple_parameters(cls):
-        """Inheritable hook for defining which parameters are simple.
-        Expected to return a list of names of parameter fields.
-
-        """
         if isinstance(cls.model, type) and issubclass(cls.model, fields.TableRow):
             return cls.model.get_simple_parameters()
         return []
@@ -1480,21 +1289,18 @@ class Actor(with_metaclass(ActorMetaClass, type('NewBase', (actions.Parametrizab
 
     @classmethod
     def get_sum_text(self, ar, sums):
-        """
-        Return the text to display on the totals row.
-        The default implementation returns "Total (N rows)".
-        """
         return str(_("Total (%d rows)") % ar.get_total_count())
 
     @classmethod
     def get_layout_aliases(cls):
         """
-
         Yield a series of (ALIAS, repl) tuples that cause a name ALIAS in a
         layout based on this actor to be replaced by its replacement `repl`.
-
         """
-        return []
+        if cls.model is None:
+            return []
+        return cls.model.get_layout_aliases()
+
 
     @classmethod
     def set_detail_layout(self, *args, **kw):
@@ -1697,14 +1503,15 @@ class Actor(with_metaclass(ActorMetaClass, type('NewBase', (actions.Parametrizab
         return list(self._actions_dict.keys())
 
     @classmethod
-    def get_toolbar_actions(self, parent):
+    def get_toolbar_actions(self, parent, user_type):
         """
         Return a list of actions for which a button should exist in the
         toolbar of the specified "parent" action.
         """
-        return [ba for ba in self.get_button_actions(parent)
-                if ba.action.show_in_bbar]
-                # if ba.action.select_rows]
+        tba = [ba for ba in self.get_button_actions(parent)
+                if ba.action.show_in_bbar and (
+                    ba.action.readonly or not self.hide_editing(user_type))]
+        return tba
 
     # @classmethod
     # def get_cell_context_actions(self, cf):
@@ -1759,8 +1566,8 @@ class Actor(with_metaclass(ActorMetaClass, type('NewBase', (actions.Parametrizab
     @classmethod
     def get_detail_elems(cls, *args):
         """
-        An optional first argument is the user interface plugin, a
-        :class:`Plugin` instance.  If this is None, will use
+        An optional first argument is the front end plugin, a
+        :class:`Plugin` instance.  If this is None, use
         :attr:`settings.SITE.kernel.default_ui
         <lino.core.kernel.Kernel.default_ui>`.
 
@@ -1876,6 +1683,8 @@ class Actor(with_metaclass(ActorMetaClass, type('NewBase', (actions.Parametrizab
         """
         Return an action request on this actor which inherits from the
         given parent request.
+
+        Deprecated. Rather call cls.request(parent=ar)
         """
         sar = cls.request(*args, **kwargs)
         sar.setup_from(ar)
@@ -1932,13 +1741,20 @@ class Actor(with_metaclass(ActorMetaClass, type('NewBase', (actions.Parametrizab
         sar = cls.request_from(ar, master_instance=obj, is_on_main_actor=False)
         p = E.div(*qs2summary(sar, sar.data_iterator, separator=cls.summary_sep,
                 max_items=cls.preview_limit))
-        if cls.insert_action is not None:
-            ir = cls.insert_action.request_from(sar)
-            if ir.get_permission():
-                btn = ir.ar2button()
-                if len(p):
-                    p.append(E.br())
-                p.append(btn)
+
+        toolbar = sar.plain_toolbar_buttons()
+        if len(toolbar):
+            p.append(E.br())
+            for b in toolbar:
+                p.append(b)
+
+        # if cls.insert_action is not None:
+        #     ir = cls.insert_action.request_from(sar)
+        #     if ir.get_permission():
+        #         btn = ir.ar2button()
+        #         if len(p):
+        #             p.append(E.br())
+        #         p.append(btn)
         return p
 
     @classmethod
@@ -1947,26 +1763,26 @@ class Actor(with_metaclass(ActorMetaClass, type('NewBase', (actions.Parametrizab
 
 
 def resolve_action(spec, action=None):
-    """
-    Return the `BoundAction` object corresponding to the given
-    specifier `spec`. The specifier can be:
-
-    - a model or a table
-    - a bound action
-    - an action instance
-    - a string of the form ``myapp.MyModel`` (i.e. resolving to a model)
-    - a string of the form ``myapp.MyModels`` (i.e. resolving to a table)
-
-    If it is an action instance, Lino will use the
-    :attr:`definiing_actor` of that action.
-
-    """
     givenspec = spec
 
-    # if isinstance(spec, string_types):
     if is_string(spec):
         site = settings.SITE
-        spec = site.models.resolve(spec)
+        # if spec.startswith('webshop.'):
+        #     print("20210320", spec, repr(site.models.resolve(spec)))
+        # spec = site.models.resolve(spec)
+        parts = spec.split('.')
+        if len(parts) < 2 or len(parts) > 3:
+            raise Exception("Invalid action specifier '{}' (must be of form `plugin_name.ClassName[.action_name]`).".format(spec))
+        app = site.models[parts[0]]
+        cls = getattr(app, parts[1])
+        if len(parts) == 2:
+            spec = cls
+        else:
+            assert action is None
+            if issubclass(cls, Actor):
+                spec = cls.get_action_by_name(parts[2])
+            else:
+                spec = getattr(cls, parts[2])
         # spec = site.actors.resolve(spec) or site.models.resolve(spec)
 
     if isinstance(spec, BoundAction):
@@ -1981,7 +1797,7 @@ def resolve_action(spec, action=None):
 
     if isinstance(spec, type) and issubclass(spec, Actor):
         if action:
-            a = spec.get_url_action(action)
+            a = spec.get_action_by_name(action)
             #~ print 20121210, a
             if a is None:
                 raise Exception(
