@@ -1,6 +1,6 @@
 # -*- coding: UTF-8 -*-
 # Copyright 2009-2020 Rumma & Ko Ltd
-# License: BSD (see file COPYING for details)
+# License: GNU Affero General Public License v3 (see file COPYING for details)
 """
 .. autosummary::
 
@@ -15,7 +15,7 @@ from lino.utils import curry
 # from .actions import Action
 
 from lino.core.permissions import (
-    make_permission_handler, make_view_permission_handler)
+    make_permission_handler, make_static_permission_handler)
 
 
 class BoundAction(object):
@@ -62,7 +62,7 @@ class BoundAction(object):
                     "for %r (required=%s) is active (settings=%s)." % (
                         self, required, os.environ['DJANGO_SETTINGS_MODULE']))
 
-        self.allow_view = curry(make_view_permission_handler(
+        self.allow_view = curry(make_static_permission_handler(
             self, action.readonly, debug_permissions, required), action)
         self._allow = curry(make_permission_handler(
             action, actor, action.readonly,
@@ -177,3 +177,6 @@ class BoundAction(object):
     def __repr__(self):
         return "<%s(%s, %r)>" % (
             self.__class__.__name__, self.actor, self.action)
+
+    def __str__(self):
+        return "{} on {}".format(self.action, self.actor)
