@@ -1,5 +1,5 @@
 # Copyright 2009-2020 Rumma & Ko Ltd
-# License: BSD (see file COPYING for details)
+# License: GNU Affero General Public License v3 (see file COPYING for details)
 
 """
 This defines the :class:`Table` class.
@@ -12,7 +12,7 @@ import datetime
 from django.conf import settings
 from django.core.exceptions import FieldDoesNotExist
 from django.db.models.fields import NOT_PROVIDED
-from django.utils.translation import ugettext_lazy as _
+from django.utils.translation import gettext_lazy as _
 
 from django.db import models
 from django.db.models.query import QuerySet
@@ -162,6 +162,10 @@ class Table(AbstractTable):
 
     """
 
+    # card_layout = """
+    # overview
+    # """
+
     debug_sql = False
 
     show_detail_navigator = True
@@ -191,7 +195,7 @@ class Table(AbstractTable):
     """If viewing the grid view on a mobile, should the grid use responsive mode
     Default: True"""
 
-    react_big_search = False
+    #react_big_search = False
     """If True will position the quick search input to the bottom of the header and have it full width.
     Default: False"""
 
@@ -280,10 +284,6 @@ class Table(AbstractTable):
     def get_actions_hotkeys(cls):
         return cls.model.get_actions_hotkeys()
 
-    @classmethod
-    def get_widget_options(cls, name, **options):
-        return cls.model.get_widget_options(name, **options)
-
     @classmethod
     def get_pk_field(self):
         return self.model._meta.pk
@@ -413,9 +413,11 @@ class Table(AbstractTable):
                     # assert direct
                     # assert not m2m
                     if fk is None:
+                        candidates = ', '.join(
+                            [f.name for f in self.model._meta.get_fields()])
                         raise Exception(
-                            "Invalid master_key {} on {}".format(
-                                self.master_key, self))
+                            "Invalid master_key '{}' for {} ({} has fields {})".format(
+                                self.master_key, self, self.model, candidates))
                     if fk.remote_field:
                         master_model = fk.remote_field.model
                     elif isinstance(fk, ChoiceListField):
@@ -546,10 +548,6 @@ class Table(AbstractTable):
             return _("You have no permission to delete this row.")
         return obj.disable_delete(ar)
 
-    @classmethod
-    def get_layout_aliases(cls):
-        return cls.model.get_layout_aliases()
-
     # @classmethod
     # def get_data_elem(self, name):
     #     """
