@@ -1,6 +1,6 @@
 # -*- coding: UTF-8 -*-
-# Copyright 2008-2020 Rumma & Ko Ltd
-# License: BSD (see file COPYING for details)
+# Copyright 2008-2021 Rumma & Ko Ltd
+# License: GNU Affero General Public License v3 (see file COPYING for details)
 
 """
 Defines extended database field classes and utility functions
@@ -11,14 +11,17 @@ import logging ; logger = logging.getLogger(__name__)
 import datetime
 from decimal import Decimal
 
+from django import http
 from django.conf import settings
 from django.db import models
-from django.utils.translation import ugettext_lazy as _
+from django.utils.translation import gettext_lazy as _
 from django.core.exceptions import ValidationError
 from django.core.exceptions import FieldDoesNotExist
 from django.db.models.fields import NOT_PROVIDED
 from django.utils.functional import cached_property
 
+from etgen.html import E, forcetext
+
 from lino.core.utils import resolve_field, full_model_name, resolve_model
 from lino.core.exceptions import ChangedAPI
 from lino.core.diff import ChangeWatcher
@@ -170,9 +173,9 @@ class PriceField(models.DecimalField):
 
 
 """
-http://stackoverflow.com/questions/454436/unique-fields-that-allow-nulls-in-django
+https://stackoverflow.com/questions/454436/unique-fields-that-allow-nulls-in-django
 answer Dec 20 '09 at 3:40 by mightyhal
-http://stackoverflow.com/a/1934764
+https://stackoverflow.com/a/1934764
 """
 
 
@@ -347,8 +350,7 @@ class RemoteField(FakeField):
         `obj`.  `ar` may be `None`, it's forwarded to the getter
         method who may decide to return values depending on it.
         """
-        m = self.func
-        return m(obj, ar)
+        return self.func(obj, ar)
 
     def __get__(self, instance, owner):
         if instance is None:
@@ -432,7 +434,7 @@ class VirtualModel:
 
 VFIELD_ATTRIBS = frozenset('''to_python choices save_form_data
   value_to_string max_length remote_field
-  max_digits verbose_name decimal_places
+  max_digits verbose_name decimal_places wildcard_data_elem
   blank'''.split())
 
 class VirtualField(FakeField):
@@ -745,7 +747,7 @@ class CharField(models.CharField):
     An extension around Django's `models.CharField`.
 
     Adds two keywords `mask_re` and `strip_chars_re` which, when using
-    the ExtJS ui, will be rendered as the `maskRe` and `stripCharsRe`
+    the ExtJS front end, will be rendered as the `maskRe` and `stripCharsRe`
     config options of `TextField` as described in the `ExtJS
     documentation
     <http://docs.sencha.com/extjs/3.4.0/#!/api/Ext.form.TextField>`__,
@@ -767,10 +769,10 @@ class CharField(models.CharField):
         self.strip_chars_re = kw.pop('strip_chars_re', None)
         self.mask_re = kw.pop('mask_re', None)
         self.regex = kw.pop('regex', None)
-        models.CharField.__init__(self, *args, **kw)
+        super(CharField, self).__init__(*args, **kw)
 
 
-class QuantityField(CharField):
+class QuantityField(models.CharField):
     """
     A field that accepts :class:`Quantity
     <lino.utils.quantities.Quantity>`, :class:`Percentage
@@ -787,44 +789,36 @@ class QuantityField(CharField):
 
     def __init__(self, *args, **kw):
         kw.setdefault('max_length', 6)
-        models.Field.__init__(self, *args, **kw)
+        super(QuantityField, self).__init__(*args, **kw)
         if self.blank and not self.null:
             raise ChangedAPI(
                 "When `blank` is True, `null` must be True as well.")
 
-        #~ models.CharField.__init__(self,*args,**kw)
-
     #~ def get_internal_type(self):
         #~ return "CharField"
 
     def to_python(self, value):
         """
         Excerpt from `Django docs
-        <https://docs.djangoproject.com/en/1.11/howto/custom-model-fields/#converting-values-to-python-objects>`__:
+        <https://docs.djangoproject.com/en/3.1/howto/custom-model-fields/#converting-values-to-python-objects>`__:
 
-            As a general rule, the method should deal gracefully with
+            As a general rule, :meth:`to_python` should deal gracefully with
             any of the following arguments:
 
-            - An instance of the correct type (e.g., Hand in our ongoing example).
+            - An instance of the correct type (e.g., `Hand` in our ongoing example).
             - A string (e.g., from a deserializer).
-            - Whatever the database returns for the column type you’re using.
+            - `None` (if the field allows `null=True`)
 
-        I'd add "Any value potentially specified for this field when instantiating
-        a model."
+        I'd add "Any value allowed for this field when instantiating a model."
 
-        >>> to_python(None)
-        >>> to_python(30)
-        >>> to_python(30L)
-        >>> to_python('')
-        >>> to_python(Decimal(0))
         """
         if isinstance(value, Decimal):
-            return value
+            return quantities.Quantity(value)
         if value:
             # try:
             if isinstance(value, str):
                 return quantities.parse(value)
-            return Decimal(value)
+            return quantities.Quantity(value)
             # except Exception as e:
             #     raise ValidationError(
             #         "Invalid value {} for {} : {}".format(value, self, e))
@@ -1043,6 +1037,7 @@ class TableRow(object):
     """Base class for everything that can be used as a table row. """
 
     _lino_default_table = None
+    _widget_options = {}
 
     hidden_columns = frozenset()
     """If specified, this is the default value for
@@ -1082,8 +1077,6 @@ class TableRow(object):
         """
         return []
 
-
-
     @classmethod
     def get_default_table(self):
         """Used internally. Lino chooses during the kernel startup, for each
@@ -1113,12 +1106,6 @@ class TableRow(object):
         #     raise Exception("Oops, {} on {} is {}".format(name, cls, v))
 
     def obj2href(self, ar, *args, **kwargs):
-        """Return a html representation of a pointer to the given database
-        object.
-
-        Examples see :ref:`obj2href`.
-
-        """
         if ar is None:
             if len(args):
                 return args[0]
@@ -1130,8 +1117,8 @@ class TableRow(object):
         a detail window.  Return `None` when no detail form exists or
         the requesting user has no permission to see it.
 
-        `ar` is the action request who asks to see a detail.
-        If the action requests's actor can be used for this model,
+        `ar` is the action request that asks to see the detail.
+        If the action request's actor can be used for this model,
         then use its `detail_action`. Otherwise use the
         `detail_action` of this model's default table.
 
@@ -1142,9 +1129,9 @@ class TableRow(object):
         :meth:`Menu.add_instance_action
         <lino.core.menus.Menu.add_instance_action>`.
 
-        Usage example: :class:`courses.Course
-        <lino_xl.lib.courses.Course>` overrides this to return
-        the detail_action depending on the ActivityLayout.
+        Usage example: :class:`courses.Course <lino_xl.lib.courses.Course>`
+        overrides this to return the detail action depending on the
+        :term:`activity layout`.
 
         """
         a = getattr(self, '_detail_action', None)
@@ -1159,6 +1146,7 @@ class TableRow(object):
                 #         ar.actor, self.__class__, ar.actor.model))
                 dt = self.__class__.get_default_table()
                 if dt is not None:
+                    # a = dt.get_request_detail_action(ar)
                     a = dt.detail_action
         if a is None or ar is None:
             return a
@@ -1168,20 +1156,50 @@ class TableRow(object):
     def get_choices_text(self, request, actor, field):
         return str(self)
 
-    def get_overview_elems(self, ar):
-        """This is expected to return a list of HTML elements to be wrapped
-        into a `<DIV>`.
+    # @fields.displayfield(_("Description"))
+    # @fields.htmlbox(_("Overview"))
+    @htmlbox()
+    def overview(self, ar):
+        if ar is None:
+            return ''
+        return E.div(*forcetext(self.get_overview_elems(ar)))
 
-        """
+    def get_overview_elems(self, ar):
         # return [ar.obj2html(self)]
         return [self.obj2href(ar)]
 
     def save_existing_instance(self, ar):
         watcher = ChangeWatcher(self)
+        # print("20210213 save_existing_instance", ar.ah, ar.rqdata, self.disabled_fields)
         ar.ah.store.form2obj(ar, ar.rqdata, self, False)
         self.full_clean()
         self.save_watched_instance(ar, watcher)
 
+    @classmethod
+    def get_layout_aliases(cls):
+        """
+
+        Yield a series of (ALIAS, repl) tuples that cause a name ALIAS in a
+        layout based on this model to be replaced by its replacement `repl`.
+
+        """
+        return []
+
+    @classmethod
+    def set_widget_options(self, name, **options):
+        # from lino.modlib.extjs.elems import FieldElement
+        # for k in options.keys():
+        #     if not hasattr(FieldElement, k):
+        #         raise Exception("Invalid widget option {}".format(k))
+        self._widget_options = dict(**self._widget_options)
+        d = self._widget_options.setdefault(name, {})
+        d.update(options)
+
+    @classmethod
+    def get_widget_options(self, name, **options):
+        options.update(self._widget_options.get(name, {}))
+        return options
+
 
 def wildcard_data_elems(model):
     """
@@ -1200,8 +1218,8 @@ def wildcard_data_elems(model):
     for f in meta.many_to_many:
         yield f
     for f in meta.private_fields:
-        #if not isinstance(f, VirtualField):
-        yield f
+        if not isinstance(f, VirtualField) or f.wildcard_data_elem:
+            yield f
     # todo: for slave in self.report.slaves
 
 
@@ -1271,7 +1289,7 @@ def pointer_factory(cls, othermodel, *args, **kw):
       case.  This difference is useful when designing reusable models.
 
     - Explicitly sets the default value for `on_delete
-      <https://docs.djangoproject.com/en/1.11/ref/models/fields/#django.db.models.ForeignKey.on_delete>`__
+      <https://docs.djangoproject.com/en/3.2/ref/models/fields/#django.db.models.ForeignKey.on_delete>`__
       to ``CASCADE`` (as required by Django 2).
     """
     if othermodel is None:
