@@ -1,19 +1,15 @@
-# Copyright 2009-2018 Rumma & Ko Ltd
-# License: BSD (see file COPYING for details)
+# Copyright 2009-2021 Rumma & Ko Ltd
+# License: GNU Affero General Public License v3 (see file COPYING for details)
 """
 Defines classes :class:`Frame` and :class:`FrameHandle`
 """
-from builtins import object
-
-import logging
-logger = logging.getLogger(__name__)
 
 # from lino.core.utils import Handle
 from lino.core import actors
 
 
 class FrameHandle(object):
-    
+
     def __init__(self, frame):
         #~ assert issubclass(frame,Frame)
         self.actor = frame
@@ -31,13 +27,13 @@ class Frame(actors.Actor):
     Base clase for actors which open a window which is neither a
     database table nor a detail form.
 
-    Example subclasses are 
+    Example subclasses are
     - :class:`lino_xl.lib.extensible.CalendarPanel`.
     - :class:`lino.modlib.awesomeuploader.UploaderPanel`.
 
     """
     _handle_class = FrameHandle
-    editable = False
+    # editable = False
     abstract = True
 
     @classmethod
