@@ -1,6 +1,6 @@
 # -*- coding: UTF-8 -*-
 # Copyright 2009-2020 Rumma & Ko Ltd
-# License: BSD (see file COPYING for details)
+# License: GNU Affero General Public License v3 (see file COPYING for details)
 
 """This defines the :class:`Kernel` class.
 
@@ -40,9 +40,9 @@ from django.apps import AppConfig
 from django.apps import apps
 from django.conf import settings
 from django.core import exceptions
-from django.utils.encoding import force_text
+from django.utils.encoding import force_str
 from django.utils.text import format_lazy
-from django.utils.translation import ugettext_lazy as _
+from django.utils.translation import gettext_lazy as _
 from django.core.exceptions import PermissionDenied
 from django.db.utils import DatabaseError
 
@@ -83,7 +83,6 @@ from .utils import resolve_fields_list
 # from .utils import djangoname
 from .utils import class_dict_items
 
-# from .inject import collect_virtual_fields
 from .fields import set_default_verbose_name
 from lino.core.requests import ActorRequest
 
@@ -105,49 +104,13 @@ class Kernel(object):
     default_ui = None  # TODO: rename this to front_end
     admin_ui = None
 
-    # _singleton_instance = None
-
-    # @classmethod
-    # def instance(cls, site):
-    #     if cls._singleton_instance is None:
-    #         cls._singleton_instance = cls(site)
-    #     elif cls._singleton_instance.site is not site:
-    #         site.logger().info("Overriding SITE instance")
-    #         cls._singleton_instance.site = site
-    #     return cls._singleton_instance
-
     def __init__(self, site):
-
         # logger.info("20140227 Kernel.__init__() a")
-
-        # from importlib import import_module
-        # # For every plugin, Lino checks whether the package contains a
-        # # module named `ui` and, if yes, imports this module. The
-        # # benefit of this is that all "Lino extensions" to the models
-        # # can be moved out of :xfile:`models.py` into a separate file
-        # # :xfile:`ui.py`.
-        # # print '\n'.join([p.app_name for p in self.installed_plugins])
-        # for p in site.installed_plugins:
-        #     # fn = dirname(inspect.getfile(p.app_module))
-        #     # fn = join(fn, 'ui.py')
-        #     try:
-        #         x = p.app_name + '.ui'
-        #         import_module(x)
-        #         logger.info("20150416 imported %s", x)
-        #     except Exception as e:
-        #     # except ImportError as e:
-        #         if str(e) != "No module named ui":
-        #             logger.warning("Failed to import %s : %s", x, e)
-        #             # raise Exception("Failed to import %s : %s" % (x, e))
-
         self.site = site
         self.GFK_LIST = []
-        # self.widgets = WidgetFactory()
-
         self.kernel_startup(site)
         # logger.info("20140227 Kernel.__init__() done")
 
-
     _code_mtime = None
 
     @property
@@ -1059,7 +1022,7 @@ def register_actors():
     - For each model we want to find out the "default table".
       The "choices table" for a foreignkey field is also currently
       simply the pointed model's default table.
-      :modattr:`_lino_default_table`
+      :attr:`lino.core.model.Model._lino_default_table`
 
     """
 
