@@ -1,15 +1,11 @@
 # -*- coding: UTF-8 -*-
-# Copyright 2009-2020 Rumma & Ko Ltd
-# License: BSD (see file COPYING for details)
+# Copyright 2009-2021 Rumma & Ko Ltd
+# License: GNU Affero General Public License v3 (see file COPYING for details)
 
 """
 Defines the classes :class:`MenuItem` and :class:`Menu`
 """
 
-from builtins import object
-
-import logging ; logger = logging.getLogger(__name__)
-
 from django.conf import settings
 
 from lino.core.utils import obj2str
@@ -51,7 +47,6 @@ class MenuItem(object):
         self.bound_action = action
         self.params = params or {}
         self.href = href
-        #~ self.request = request
         self.instance = instance
         self.javascript = javascript
         self.help_text = help_text
@@ -154,25 +149,38 @@ class MenuItem(object):
         return js
 
 
-def create_item(spec, action=None, help_text=None, **kw):
+def create_item(unused, spec, action=None, help_text=None, **kw):
+    """Create a menu item according to the specified arguments.
+
+    TODO: explain the arguments and their meaning.
+
     """
-    """
-    a = resolve_action(spec, action)
-    if help_text is None:
-        if a == a.actor.default_action:
-            help_text = a.actor.help_text or a.action.help_text
-        else:
-            help_text = a.action.help_text
-    if help_text is not None:
-        kw.update(help_text=help_text)
-    kw.update(action=a)
+    if isinstance(spec, dict):
+        assert help_text is None
+        assert action is None
+        kw.update(spec)
+        action_spec = spec.get('action', None)
+        if action_spec is not None:
+            kw.update(action=resolve_action(action_spec))
+    else:
+        a = resolve_action(spec, action)
+        # if str(spec).startswith("webshop"):
+        #     print("20210319", spec, action, a)
+        kw.update(action=a)
+        if help_text is None:
+            if a is a.actor.default_action:
+                help_text = a.actor.help_text or a.action.help_text
+            else:
+                help_text = a.action.help_text
+        if help_text is not None:
+            kw.update(help_text=help_text)
     return MenuItem(**kw)
 
 
 class Menu(MenuItem):
     """
-    Represents a menu. A menu is conceptually a :class:`MenuItem`
-    which contains other menu items.
+    Represents a menu. A menu is a :class:`MenuItem`
+    that contains other menu items.
     """
 
     avoid_lonely_items = False
@@ -248,12 +256,16 @@ class Menu(MenuItem):
         self.items = newitems
 
     def add_action(self, *args, **kw):
-        mi = create_item(*args, **kw)
+        """
+        Add a menu item to this menu.
+        All arguments are forwarded to :func:`create_item`.
+        """
+        mi = create_item(None, *args, **kw)
         return self.add_item_instance(mi)
 
     def add_instance_action(self, obj, **kw):
         """
-        Add an action which displays the given database object instance in
+        Add an action that displays the given database object instance in
         a detail form for editing.
 
         Used e.g. for the :menuselection`Configure --> System --> Site
@@ -304,7 +316,6 @@ class Menu(MenuItem):
                 return
             if not mi.bound_action.get_view_permission(self.user_type):
                 return
-            #~ logger.info("20130129 _add_item %s for %s",mi.label,self.user_type)
         if mi.name is not None:
             old = self.items_dict.get(mi.name)
             if old is not None:
