@@ -1,6 +1,6 @@
 # -*- coding: UTF-8 -*-
 # Copyright 2009-2020 Rumma & Ko Ltd
-# License: BSD (see file COPYING for details)
+# License: GNU Affero General Public License v3 (see file COPYING for details)
 """
 Defines :class:`HtmlRenderer` and :class:`TextRenderer`.
 """
@@ -13,11 +13,11 @@ import rstgen
 from django.conf import settings
 from django.db import models
 from django.db.models.fields import NOT_PROVIDED
-# from django.utils.encoding import force_text
+# from django.utils.encoding import force_str
 from django.utils.text import format_lazy
 
 
-from django.utils.translation import ugettext as _
+from django.utils.translation import gettext as _
 from django.utils.translation import get_language
 
 from etgen.html2rst import RstTable
@@ -182,10 +182,9 @@ class HtmlRenderer(Renderer):
         Silently ignores the parameters `stripped` and `header_links`
         since for HTML these options have no meaning.
         """
-        if display_mode is None:
-            display_mode = ar.actor.display_mode
-        # if ar.actor.master is not None and not nosummary:
         if not nosummary:
+            if display_mode is None:
+                display_mode = ar.actor.display_mode
             if display_mode == 'summary':
                 yield ar.actor.get_table_summary(ar.master_instance, ar)
                 return
@@ -225,23 +224,24 @@ class HtmlRenderer(Renderer):
     def href_to(self, ar, obj, text=None):
         h = self.obj2url(ar, obj)
         if h is None:
-            # return escape(force_text(obj))
+            # return escape(force_str(obj))
             return escape(str(obj))
         uri = self.js2url(h)
-        # return self.href(uri, text or force_text(obj))
+        # return self.href(uri, text or force_str(obj))
         return self.href(uri, text or str(obj))
 
     def href_to_request(self, ar, tar, text=None, **kw):
         """
-
-        Return a string with an URL which would run the given target request
+        Return a string with an URL that would run the given target request
         `tar`."""
+        if text is None:
+            text = tar.get_title()
 
-        # TODO: remove the `ar` as it is not needed.
+        if ar and ar.actor and ar.actor.hide_navigator:
+            return text
 
         uri = self.js2url(self.request_handler(tar))
-        return self.href_button_action(
-            tar.bound_action, uri, text or tar.get_title(), **kw)
+        return self.href_button_action(tar.bound_action, uri, text, **kw)
 
     def href_button_action(
             self, ba, url, text=None, title=None, icon_name=NOT_PROVIDED, **kw):
@@ -387,7 +387,7 @@ class HtmlRenderer(Renderer):
 
         """
         if text is None:
-            # text = (force_text(obj),)
+            # text = (force_str(obj),)
             text = (str(obj),)
         elif isinstance(text, str) or iselement(text):
             text = (text,)
@@ -480,7 +480,9 @@ class HtmlRenderer(Renderer):
         try:
             for item in forcetext(story):
                 # print("20180907 {}".format(item))
-                if iselement(item):
+                if isinstance(item, str):
+                    elems.append(item)
+                elif iselement(item):
                     # 20200501 elems.append(item)
                     elems.append(tostring(item))
                 elif isinstance(item, type) and issubclass(item, Actor):
@@ -497,17 +499,19 @@ class HtmlRenderer(Renderer):
                     # 20200501 elems.append(self.show_story(ar, item.actor.get_story(None, ar), **kwargs))
                     elems += [tostring(e) for e in self.show_story(ar, item.actor.get_story(None, ar), **kwargs)]
                 elif isinstance(item, DashboardItem):
-                    html = self.show_story(ar, item.render(ar), **kwargs)
-                    # 20200501 if len(html):
-                    #     elems.append(E.div(
-                    #     html,
-                    #     CLASS="dashboard-item " + item.actor.actor_id.replace(".","-") if getattr(item, "actor", False) else ""
-                    if html: # should always be a string, never a list
-                        if hasattr(item, "actor"):
-                            css_class = "dashboard-item " + item.actor.actor_id.replace(".","-")
-                        else:
-                            css_class = ''
-                        elems.append('<div class="{}">{}</div>'.format(css_class, html))
+                    elems.extend(item.render(ar, **kwargs))
+                    # html = self.show_story(ar, item.render(ar), **kwargs)
+                    # elems.append(html)
+                    # # 20200501 if len(html):
+                    # #     elems.append(E.div(
+                    # #     html,
+                    # #     CLASS="dashboard-item " + item.actor.actor_id.replace(".","-") if getattr(item, "actor", False) else ""
+                    # if html: # should always be a string, never a list
+                    #     if hasattr(item, "actor"):
+                    #         css_class = "dashboard-item " + item.actor.actor_id.replace(".","-")
+                    #     else:
+                    #         css_class = ''
+                    #     elems.append('<div class="{}">{}</div>'.format(css_class, html))
 
                 elif isiterable(item):
                     elems.append(self.show_story(ar, item, **kwargs))
@@ -525,8 +529,9 @@ class HtmlRenderer(Renderer):
 
     def show_menu(self, ar, mnu, level=1):
         """
-        Render the given menu as an HTML element.
-        Used for writing test cases.
+        Render the given menu as an HTML etree element.
+
+        Used by bootstrap3 front end.
         """
         if not isinstance(mnu, Menu):
             assert isinstance(mnu, MenuItem)
@@ -572,9 +577,11 @@ class HtmlRenderer(Renderer):
     def add_help_text(self, kw, help_text, title, datasource, fieldname):
         pass
 
+
 class TextRenderer(HtmlRenderer):
     """
-    Lino renderer which renders tables as reStructuredText to stdout.
+    Renders things as reStructuredText to stdout.
+
     Used for doctests and console output.
     See also :class:`TestRenderer`.
     """
@@ -592,46 +599,6 @@ class TextRenderer(HtmlRenderer):
     #     # return str(actor)+"/"+str(pk)
     #     return "Detail"  # many doctests depend on this
 
-    def menu2rst(self, ar, mnu, level=1):
-        """Used by :meth:`show_menu`."""
-
-        if not isinstance(mnu, Menu):
-            return str(mnu.label)
-
-        has_submenus = False
-        for i in mnu.items:
-            if isinstance(i, Menu):
-                has_submenus = True
-        items = [self.menu2rst(ar, mi, level + 1) for mi in mnu.items]
-        if has_submenus:
-            s = rstgen.ul(items).strip() + '\n'
-            if mnu.label is not None:
-                s = str(mnu.label) + ' :\n\n' + s
-        else:
-            s = ', '.join(items)
-            if mnu.label is not None:
-                s = str(mnu.label) + ' : ' + s
-        return s
-
-    def show_menu(self, ar, mnu, stripped=True, level=1):
-        """
-        Render the given menu as a reStructuredText
-        formatted bullet list.
-        Called from :meth:`lino.core.requests.BaseRequest.show_menu`.
-
-        :stripped: remove lots of blanklines which are necessary for
-                   reStructuredText but disturbing in a doctest
-                   snippet.
-
-        """
-        s = self.menu2rst(ar, mnu, level)
-        if stripped:
-            for ln in s.splitlines():
-                if ln.strip():
-                    print(ln)
-        else:
-            print(s)
-
     def show_table(self, *args, **kwargs):
         for ln in self.table2story(*args, **kwargs):
             print(ln)
@@ -716,8 +683,7 @@ class TextRenderer(HtmlRenderer):
                     ar = item.default_action.request(parent=ar)
                     self.show_table(ar, stripped=stripped, **kwargs)
                 elif isinstance(item, DashboardItem):
-                    self.show_story(
-                        ar, item.render(ar), stripped, **kwargs)
+                    self.show_story(ar, item.get_story(ar), stripped, **kwargs)
                 elif isinstance(item, TableRequest):
                     self.show_table(item, stripped=stripped, **kwargs)
                     # print(item.table2rst(*args, **kwargs))
@@ -738,7 +704,7 @@ class TextRenderer(HtmlRenderer):
         """Used by :meth:`lino.core.requests.BaseRequest.obj2str`.
         """
         if text is None:
-            # text = force_text(obj)
+            # text = force_str(obj)
             text = str(obj)
         # return "**{0}**".format(text)
         return settings.SITE.obj2text_template.format(text)
@@ -784,45 +750,10 @@ class JsRenderer(HtmlRenderer):
 
 
     def goto_instance(self, ar, obj, detail_action=None, **kw):
-        """Ask the client to display a detail window on the given
-        record. The client might ignore this if Lino does not know a
-        detail window.
-
-        This is a utility wrapper around :meth:`set_response` which sets
-        either `data_record` or a `record_id`.
-
-        Usually `data_record`, except if it is a `file upload
-        <https://docs.djangoproject.com/en/3.1/topics/http/file-uploads/>`_
-        where some mysterious decoding problems (:blogref:`20120209`)
-        force us to return a `record_id` which has the same visible
-        result but using an additional GET.
-
-        If the calling window is a detail on the same table, then it
-        should simply get updated to the given record. Otherwise open a
-        new detail window.
-
-        If the detail layout of the current actor can be used for the
-        object to be displayed, we don't want to open a new detail
-        window.
-
-        This calls :meth:`obj.get_detail_action
-        <lino.core.model.Model.get_detail_action>`.
-
+        """Instruct the client to display a detail window on the given
+        record.
         """
         # print("20201230a goto_instance", ar.actor, detail_action)
-        if ar.actor is not None:
-            da = detail_action or obj.get_detail_action(ar)
-            if da is None:
-                return
-            # print("20201230b goto_instance", da.actor, ar.actor)
-            if da.actor == ar.actor:
-                ar.set_response(detail_handler_name=da.full_name())
-                if ar.actor.handle_uploaded_files is not None:
-                    ar.set_response(record_id=obj.pk)
-                else:
-                    ar.set_response(
-                        data_record=ar.elem2rec_detailed(obj))
-                return
         js = self.instance_handler(ar, obj, detail_action)
         kw.update(eval_js=js)
         ar.set_response(**kw)
@@ -893,7 +824,7 @@ class JsRenderer(HtmlRenderer):
         if ba is None:
             return None
         if not ba.get_row_permission(ar, obj, None):
-            # fixes #3857
+            # fixes #3857 (Lino links to a ticket and then says it doesn't exist)
             return None
         if ar.permalink_uris:
             return self.get_detail_url(ar, ba.actor, obj.pk)
