@@ -1,6 +1,6 @@
 # -*- coding: UTF-8 -*-
 # Copyright 2009-2020 Rumma & Ko Ltd
-# License: BSD (see file COPYING for details)
+# License: GNU Affero General Public License v3 (see file COPYING for details)
 """
 Defines :class:`JsCacheRenderer`.
 """
@@ -87,19 +87,27 @@ class JsCacheRenderer():
         self.form_panels = set()
         self.param_panels = set()
         self.action_param_panels = set()
+        self.other_panels = set()
 
         def add(res, collector, fl, formpanel_name):
             # res: an actor class or action instance
             # collector: one of form_panels, param_panels or
             # action_param_panels
             # fl : a FormLayout
+            # if str(fl).endswith("Given"):
+            #     print("20210223 add", fl)
+            # if str(res).endswith("MyCoursesGiven"):
+            #     print("20210223 gonna add {} for {}".format(fl, res))
             if fl is None:
                 return
             if fl._datasource is None:
+                # raise Exception("20210223 {}".format(res))
                 return  # 20130804
 
             if fl._datasource != res:
-                fl._other_datasources.add(res)
+                fl.add_datasource(res)
+                # if str(res).endswith("MyCoursesGiven"):
+                #     print("20210223 added", fl._other_datasources)
                 # if str(res).startswith('newcomers.AvailableCoaches'):
                 #     logger.info("20150716 %s also needed by %s", fl, res)
                 # if str(res) == 'courses.Pupils':
@@ -133,6 +141,10 @@ class JsCacheRenderer():
             # if res.parameters is not None:
             add(res, self.param_panels, res.params_layout,
                 "%s.ParamsPanel" % res)
+            add(res, self.other_panels, res.card_layout,
+                "%s.CardsPanel" % res)
+            add(res, self.other_panels, res.item_layout,
+                "%s.ItemsPanel" % res)
 
             for ba in res.get_actions():
                 if ba.action.parameters:
@@ -181,8 +193,6 @@ class JsCacheRenderer():
 
         settings.SITE.on_each_app('setup_site_cache', force)
 
-        settings.SITE.makedirs_if_missing(
-            os.path.join(settings.MEDIA_ROOT, 'upload'))
         settings.SITE.makedirs_if_missing(
             os.path.join(settings.MEDIA_ROOT, 'webdav'))
 
