@@ -1,5 +1,5 @@
 # Copyright 2009-2020 Rumma & Ko Ltd
-# License: BSD (see file COPYING for details)
+# License: GNU Affero General Public License v3 (see file COPYING for details)
 """Defines the "store" and its "fields" .
 
 During startup, Lino instantiates a "store" and its "fields" (aka
@@ -31,8 +31,8 @@ import datetime
 from django.conf import settings
 from django.db import models
 from django.core import exceptions
-from django.utils.translation import ugettext_lazy as _
-from django.utils.encoding import force_text
+from django.utils.translation import gettext_lazy as _
+from django.utils.encoding import force_str
 from lino import AFTER17
 
 from lino.utils.jsgen import py2js
@@ -113,14 +113,14 @@ class StoreField(object):
     def value2list(self, ar, v, l, row):
         return l.append(v)
 
-    def value2dict(self, ar, v, d, row):
-        d[str(self.name)] = v
+    def value2dict(self, ar, k, v, d, row):
+        d[k] = v
 
     # def value2odt(self,ar,v,tc,**params):
         # """
         # Add the necessary :term:`odfpy` element(s) to the containing element `tc`.
         # """
-        # params.update(text=force_text(v))
+        # params.update(text=force_str(v))
         # tc.addElement(odf.text.P(**params))
 
     def parse_form_value(self, v, obj):
@@ -138,11 +138,13 @@ class StoreField(object):
         - setting a CharField to '' will store either '' or None,
           depending on whether the field is nullable or not.
         - sales.Invoice.number may be blank
+        - setting a Person.city to blank must not create a new Place "Select a Place..."
 
         """
         v = self.extract_form_data(instance, post_data)
-        # logger.info("20180712 %s.form2obj() %s = %r",
-        #             self.__class__.__name__, self.name, v)
+        # if self.name in 'nationality city':
+        #     logger.info("20210213 %s.form2obj() %s = %r from %s",
+        #                 self.__class__.__name__, self.name, v, post_data)
         if v is None:
             # means that the field wasn't part of the submitted form. don't
             # touch it.
@@ -194,7 +196,7 @@ class StoreField(object):
         Return a plain textual representation as a unicode string
         of the given value `v`.   Note that `v` might be `None`.
         """
-        return force_text(v)
+        return force_str(v)
 
 
 class RelatedMixin(object):
@@ -253,10 +255,10 @@ class ComboStoreField(StoreField):
         l += [text, value]
 
     # def obj2dict(self,request,obj,d):
-    def value2dict(self, ar, v, d, row):
+    def value2dict(self, ar, k, v, d, row):
         value, text = self.get_value_text(ar, v, row)
-        d[str(self.name)] = text
-        d[str(self.name + constants.CHOICES_HIDDEN_SUFFIX)] = value
+        d[k] = text
+        d[k + constants.CHOICES_HIDDEN_SUFFIX] = value
 
     def get_value_text(self, ar, v, obj):
         # v = self.full_value_from_object(None,obj)
@@ -296,33 +298,41 @@ class ForeignKeyStoreField(RelatedMixin, ComboStoreField):
             #return (v.pk, str(v))
 
     def extract_form_data(self, obj, post_data):
-        # returns a tuple with both values
         # logger.info("20130128 ComboStoreField.extract_form_data %s",self.name)
-        text = post_data.get(self.name, None)
         pk = post_data.get(self.name + constants.CHOICES_HIDDEN_SUFFIX, None)
-        if pk is None and text is None:
+        if pk is None:
+            # for FK fields the fooHidden must be provided by
+            # the form, otherwise we consider the field as not included in the form.
+            # This is not an error because not all forms submit all fields.
+            # logger.info("20210214 incomplete form data for %s", self)
+            # raise Exception("20210214 incomplete form data for %s" % self)
             return
+
+        if not pk:
+            return ''
+            # return an empty string because None would mean to not touch the
+            # field. Field.blank and Field.none are handled later.
+
         relto_model = self.get_rel_to(obj)
         if not relto_model:
+            raise Warning("extract_form_data found no relto_model for %s" % self)
             # logger.info("20111209 get_value_text: no relto_model")
-            return
+            # return
 
-        if pk is not None:
-            # print("20200901 parse_form_value", v, obj)
-            try:
-                return relto_model.objects.get(pk=pk)
-            except ValueError:
-                pass
-            except relto_model.DoesNotExist:
-                pass
-            # don't return here because for learning comboboxes, extjs sets both
-            # partner and partnerHidden to the text
+        # print("20200901 parse_form_value", v, obj)
+        try:
+            return relto_model.objects.get(pk=pk)
+        except (ValueError, relto_model.DoesNotExist):
+            # For learning comboboxes, extjs sets fooHidden to contain the
+            # text that did not find a matching pk.
 
-        if text and obj is not None:
-            ch = obj.__class__.get_chooser_for_field(self.field.name)
-            if ch and ch.can_create_choice:
-                # print("20200901 can_create_choice", obj, v)
-                return ch.create_choice(obj, text)
+            if obj is not None:
+                ch = obj.__class__.get_chooser_for_field(self.field.name)
+                if ch and ch.can_create_choice:
+                    # print("20200901 can_create_choice", obj, v)
+                    return ch.create_choice(obj, pk)
+
+        raise Warning("Got invalid non-empty pk {} for {}".format(pk, self))
 
     def parse_form_value(self, v, obj):
         """Convert the form field value (expected to contain a primary key)
@@ -394,7 +404,7 @@ class VirtStoreField(StoreField):
         # 20130130 self.value2num = delegate.value2num
         # 20130130 self.value2html = delegate.value2html
         self.value2list = delegate.value2list
-        self.value2dict = delegate.value2dict
+        self.value2dict = delegate.value2dict # 20210516
         self.format_value = delegate.format_value
         self.extract_form_data = delegate.extract_form_data
         # 20130130 self.format_sum = delegate.format_sum
@@ -409,7 +419,7 @@ class VirtStoreField(StoreField):
         self.delegate = delegate
 
     def __repr__(self):
-        return '(virtual)' + self.delegate.__class__.__name__ + ' ' + self.name
+        return "(virtual){} '{}'".format(self.delegate.__class__.__name__, self.name)
 
     def full_value_from_object(self, obj, ar=None):
         # 20150218 : added new rule that virtual fields are never
@@ -425,6 +435,15 @@ class VirtStoreField(StoreField):
             return None
         return self.vf.value_from_object(obj, ar)
 
+    # 20210516
+    # def value2dict(self, ar, v, d, row):
+    #     d[self.name] = v
+    # def value2dict(self, ar, v, d, row):
+    #     d2 = {}
+    #     self.delegate.value2dict(ar, v, d2, row)
+    #     for k, v in d2.items():
+    #         d[self.name + k] = v
+
 
 class RequestStoreField(StoreField):
     """
@@ -445,8 +464,8 @@ class RequestStoreField(StoreField):
     def value2list(self, ar, v, l, row):
         return l.append(self.format_value(ar, v))
 
-    def value2dict(self, ar, v, d, row):
-        d[str(self.name)] = self.format_value(ar, v)
+    def value2dict(self, ar, k, v, d, row):
+        d[k] = self.format_value(ar, v)
         # d[self.options['name']] = self.format_value(ui,v)
         # d[self.field.name] = v
 
@@ -588,16 +607,17 @@ class DisableEditingStoreField(SpecialStoreField):
     method on that row.
     New feature since `/blog/2011/0830`
     """
-    name = str('disable_editing')
+    name = 'disable_editing'
 
     def full_value_from_object(self, obj, ar=None):
         # import pdb; pdb.set_trace()
         actor = self.store.actor
-        if actor.update_action is None:
-            # print 20120601, self.store.actor, "update_action is None"
-            return True  # disable editing if there's no update_action
         if ar is None:
             return True
+        # if actor.update_action is None:
+        if actor.hide_editing(ar.get_user().user_type):
+            # print 20120601, self.store.actor, "update_action is None"
+            return True  # disable editing if there's no update_action
         v = actor.get_row_permission(
             obj, ar, actor.get_row_state(obj), actor.update_action)
         # if str(actor).endswith('.RegisterNewUser'):
@@ -632,7 +652,7 @@ class BooleanStoreField(StoreField):
         return constants.parse_boolean(v)
 
     def format_value(self, ar, v):
-        return force_text(iif(v, _("Yes"), _("No")))
+        return force_str(iif(v, _("Yes"), _("No")))
 
 
 class DisplayStoreField(StoreField):
@@ -869,19 +889,16 @@ def create_atomizer(holder, fld, name):
     if isinstance(fld, fields.RemoteField):
         """
         Hack: we create a StoreField based on the remote field,
-        then modify its behaviour.
+        then modify some of its behaviour.
         """
         sf = create_atomizer(holder, fld.field, fld.name)
         # print("20180712 create_atomizer {} from {}".format(sf, fld.field))
 
-        def value_from_object(sf, obj, ar=None):
-            m = fld.func
-            return m(obj, ar)
+        def value_from_object(unused, obj, ar=None):
+            return fld.func(obj, ar)
 
-        def full_value_from_object(sf, obj, ar=None):
-            m = fld.func
-            v = m(obj, ar)
-            return v
+        def full_value_from_object(unused, obj, ar=None):
+            return fld.func(obj, ar)
 
         def set_value_in_object(sf, ar, instance, v):
             # print("20180712 {}.set_value_in_object({}, {})".format(
@@ -1008,7 +1025,7 @@ class ParameterStore(BaseStore):
             v = pv.get(fld.name, None)
             # if "__" in fld.name:
             #     print("20200430", fld, v)
-            fld.value2dict(ar, v, d, None)
+            fld.value2dict(ar, fld.name, v, d, None)
             # try:
             #     v = pv.get(fld.name, None)
             #     fld.value2dict(ar, v, d, None)
@@ -1083,8 +1100,8 @@ class Store(BaseStore):
         self.all_fields = []
         self.list_fields = []
         self.detail_fields = []
-        self.detail_fields = []
         self.card_fields = []
+        self.item_fields = []
         self.primary_keys = set([])
 
         def addfield(sf):
@@ -1110,6 +1127,11 @@ class Store(BaseStore):
             dh = form.get_layout_handle(settings.SITE.kernel.default_ui)
             self.collect_fields(self.card_fields, dh)
 
+        form = rh.actor.item_layout
+        if form:
+            dh = form.get_layout_handle(settings.SITE.kernel.default_ui)
+            self.collect_fields(self.item_fields, dh)
+
         if self.pk is not None:
             self.pk_index = 0
             found = False
@@ -1137,14 +1159,15 @@ class Store(BaseStore):
         # if not issubclass(rh.report,dbtables.Table):
             # addfield(RecnoStoreField(self))
 
-        if rh.actor.editable:
+        actor_editable = True  # not rh.actor.hide_editing(None)
+        if actor_editable:
             self._disabled_fields_storefield = DisabledFieldsStoreField(self)
             addfield(self._disabled_fields_storefield)
             # NB what about disabled actions on non-editable actor?
 
         # addfield(DisabledActionsStoreField(self))
 
-        if rh.actor.editable:
+        if actor_editable:
             addfield(DisableEditingStoreField(self))
 
         if rh.actor.get_row_classes is not None:
@@ -1161,6 +1184,7 @@ class Store(BaseStore):
         self.list_fields = tuple(self.list_fields)
         self.detail_fields = tuple(self.detail_fields)
         self.card_fields = tuple(self.card_fields)
+        self.item_fields = tuple(self.item_fields)
 
     def collect_fields(self, fields, *layouts):
         """`fields` is a pointer to either `self.detail_fields` or
@@ -1214,8 +1238,8 @@ class Store(BaseStore):
         Store the `form_values` into the `instance` by calling
         :meth:`form2obj` for every store field.
         """
-        # logger.info("20180813")
         disabled_fields = set(self.actor.get_disabled_fields(instance, ar))
+        # logger.info("20210213 form2obj %s %s", disabled_fields, [f.name for f in self.all_fields])
         changed_triggers = []
         for f in self.all_fields:
             if f.name not in disabled_fields:
@@ -1286,13 +1310,16 @@ class Store(BaseStore):
                 continue
             # logger.info("20140429 Store.row2dict %s", fld)
             v = fld.full_value_from_object(row, ar)
-            fld.value2dict(ar, v, d, row)
+            fld.value2dict(ar, fld.name, v, d, row)
             # logger.info("20140429 Store.row2dict %s -> %s", fld, v)
         #     if "households_" in fld.name:
         #         print("20181023 {}".format(fld))
         # print("20181023 row2dict {}".format(fields))
         return d
 
+    # def file2url(self, ar, row):
+    #     return settings.SITE.build_media_url(row.file.name)
+
     # def row2odt(self,request,fields,row,sums):
         # for i,fld in enumerate(fields):
             # if fld.field is not None:
