@@ -1,6 +1,6 @@
 # -*- coding: UTF-8 -*-
 # Copyright 2009-2020 Rumma & Ko Ltd
-# License: BSD (see file COPYING for details)
+# License: GNU Affero General Public License v3 (see file COPYING for details)
 
 """Defines the classes :class:`AbstractTable` and
 :class:`VirtualTable`.
@@ -13,7 +13,7 @@ import yaml
 
 from django.db import models
 from django.conf import settings
-from django.utils.translation import ugettext_lazy as _
+from django.utils.translation import gettext_lazy as _
 
 from lino.core import actors
 from lino.core import actions
@@ -170,38 +170,7 @@ class AbstractTable(actors.Actor):
 
     """
 
-    # column_names_m = None
-    # """An optional alternative for :attr:`column_names` to use when
-    # :attr:`mobile_view <lino.core.site.Site.mobile_view>` is True.
-    #
-    # """
-
     column_names = '*'
-    """A string that describes the list of columns of this table.
-
-    Lino will automatically create a
-    :class:`lino.core.layouts.ColumnsLayout` from this.
-    This string must not contain any newline characters because a
-    ColumnsLayout's `main` panel descriptor must be horizontal.
-
-    Default value is ``'*'``. Where all columns are included.
-    This wildcard character means "all columns which have not been
-    named explicitly can be selected by the user and inserted at
-    this point". It can be combined with explicitly specified names.
-
-    For example::
-
-      column_names = "name owner * date"
-
-    specifies that `name` and `owner` come first, followed by inserted
-    columns and finally by `date`.
-
-    If ``'*'`` is not present in the string only explicitly named
-    columns will be available.
-
-    See also :meth:`setup_column` and :meth:`get_column_names`.
-
-    """
 
     tablet_columns = None
     """
@@ -224,17 +193,17 @@ class AbstractTable(actors.Actor):
     will not pop-in.
     """
 
-    start_at_bottom = False
-    """Set this to `True` if you want your table to *start at the
-    bottom*.  Unlike reverse ordering, the rows remain in their
-    natural order, but when we open a grid on this table, we want it
-    to start on the last page.
-
-    Use cases would be :class:`lino_xl.lib.sales.InvoicesByJournal` and
-    :class:`lino_xl.lib.ledger.InvoicesByJournal` but the result is not yet
-    satisfying.
-
-    """
+    # start_at_bottom = False
+    # """Set this to `True` if you want your table to *start at the
+    # bottom*.  Unlike reverse ordering, the rows remain in their
+    # natural order, but when we open a grid on this table, we want it
+    # to start on the last page.
+    #
+    # Use cases would be :class:`lino_xl.lib.sales.InvoicesByJournal` and
+    # :class:`lino_xl.lib.ledger.InvoicesByJournal` but the result is not yet
+    # satisfying.
+    #
+    # """
 
     group_by = None
     """
@@ -273,32 +242,6 @@ class AbstractTable(actors.Actor):
     """
 
     preview_limit = settings.SITE.preview_limit
-    """The maximum number of rows to fetch when this table is being
-    displayed in "preview mode", i.e. (1) as a slave table in a detail
-    window or (2) as a dashboard item (:meth:`get_dashboard_items
-    <lino.core.site.Site.get_dashboard_items>`) in
-    :xfile:`admin_main.html`.
-
-    The default value for this is the :attr:`preview_limit
-    <lino.core.site.Site.preview_limit>` class attribute of your
-    :class:`Site <lino.core.site.Site>`, which itself has a hard-coded
-    default value of 15 and which you can override in your
-    :xfile:`settings.py`.
-
-    If you set this to `0`, preview requests for this table will
-    request all rows.  Since preview tables usually have no paging
-    toolbar, that's theoretically what we want (but can lead to waste
-    of performance if there are many rows).
-
-    In React if set to `0` the paging toolbar which usually is
-    present in the detail view, will be removed, as it has no use, as
-    all rows wil be displayed.
-
-    Test case and description in the tested docs of :ref:`cosi`.
-
-
-
-    """
 
     row_height = 1
     """
@@ -340,23 +283,6 @@ class AbstractTable(actors.Actor):
     """
 
     hidden_columns = frozenset()
-    """If given, this is specifies the data elements that should be
-    hidden by default when rendering this table.  Example::
-
-      hidden_columns = "long_name expected_date"
-
-    **Value** : Application code should specify this as a *single
-    string* containing a space-separated list of field names.  Lino
-    will automatically resolve this during server startup using
-    :func:`lino.core.utils.fields_list`.  The runtime value of this
-    attribute is a *set of strings*, each one the name of a data
-    element. Defaults to an empty set.
-
-    **Inheritance** : Note that this can be specified either on a
-    :class:`Model` or on a :class:`Table`.  Lino will make a union of
-    both.
-
-    """
 
     form_class = None
     help_url = None
@@ -392,23 +318,10 @@ class AbstractTable(actors.Actor):
     """
 
     display_mode = 'grid'
-    """
-    How to display this table when it is a slave in a detail
-    window. Must be one of the following values:
-
-    - `'grid'` (default) to render as a grid.
-    - `'summary'` to render a summary in a HtmlBoxPanel.
-    - `'html'` to render plain html a HtmlBoxPanel.
-    - `'cards'` to render a defined layout as a grid of cards (react only)
-    - `'list'` to render a defined layout as a list of cards (react only
-
-    See :doc:`/dev/table_summaries`.
-
-    """
 
     max_render_depth = 2
     """
-    Used to limit the rendering of slave card views. 
+    Used to limit the rendering of slave card views.
     """
 
     hide_if_empty = False
@@ -441,11 +354,6 @@ class AbstractTable(actors.Actor):
 
     """
 
-    grid_configs = []
-    """
-    Will be filled during :meth:`lino.core.table.Table.do_setup`.
-    """
-
     order_by = None
     """If specified, this must be a tuple or list of field names which
 will be passed to Django's `order_by
@@ -570,8 +478,6 @@ method in order to sort the rows of the queryset.
 
     @classmethod
     def get_default_action(cls):
-        #~ return actions.BoundAction(cls,cls.grid)
-        #~ return 'grid'
         return actions.ShowTable()
 
     @classmethod
@@ -606,26 +512,6 @@ method in order to sort the rows of the queryset.
             yield cc
         #~ return []
 
-    @classmethod
-    def save_grid_config(self, index, data):
-        raise Exception("20130710")
-        if len(self.grid_configs) == 0:
-            gc = GridConfig(self, data, '%s.gc' % self)
-            self.grid_configs.append(gc)
-        else:
-            gc = self.grid_configs[index]
-        gc.data = data
-        gc.validate()
-        #~ self.grid_configs[index] = gc
-        return gc.save_config()
-        #~ filename = self.get_grid_config_file(gc)
-        #~ f = open(filename,'w')
-        # ~ f.write("# Generated file. Delete it to restore default configuration.\n")
-        #~ d = dict(grid_configs=self.grid_configs)
-        #~ f.write(yaml.dump(d))
-        #~ f.close()
-        #~ return "Grid Config has been saved to %s" % filename
-
     @classmethod
     def get_filter_kw(self, ar, **kw):
         """
@@ -721,12 +607,15 @@ method in order to sort the rows of the queryset.
         To be combined with the `show` management command.
         """
         settings.SITE.startup()
+        ba = None
         if pk is not None:
             if an is None:
-                an = self.default_elem_action_name
+                # an = self.default_elem_action_name
+                ba = self.detail_action
         elif an is None:
             an = self.default_list_action_name
-        ba = self.get_action_by_name(an)
+        if ba is None:
+            ba = self.get_action_by_name(an)
         #~ print ba
         if pk is None:
             ar = self.request(action=ba)
