@@ -1,5 +1,5 @@
-# Copyright 2010-2020 Rumma & Ko Ltd
-# License: BSD (see file COPYING for details)
+# Copyright 2010-2021 Rumma & Ko Ltd
+# License: GNU Affero General Public License v3 (see file COPYING for details)
 """
 A collection of utilities which require Django settings to be
 importable.
@@ -15,7 +15,7 @@ from django.db.models import Q
 from django.core.exceptions import FieldDoesNotExist
 from django.utils.functional import lazy
 from importlib import import_module
-from django.utils.translation import ugettext as _
+from django.utils.translation import gettext as _
 from django.conf import settings
 from django.core import exceptions
 from django.http import QueryDict
@@ -111,7 +111,7 @@ def is_devserver():
 
     Thanks to Aryeh Leib Taurog in `How can I tell whether my Django
     application is running on development server or not?
-    <http://stackoverflow.com/questions/1291755>`_
+    <https://stackoverflow.com/questions/1291755>`_
 
     My additions:
 
@@ -240,9 +240,10 @@ def models_by_base(base, toplevel_only=False):
     base class.
 
     If `toplevel_only` is True, then do not include MTI children.
+    See :ref:`tested.core_utils` for more explanations.
 
-    Changed 2015-11-03: The list is sorted alphabetically using
-    :func:`full_model_name` because anyway the sort order was
+    The list is sorted alphabetically using :func:`full_model_name`.
+    Before 2015-11-03 it was
     unpredictable and changed between Django versions.
 
     """
@@ -293,11 +294,11 @@ def inrange_filter(fld, rng, **kw):
 
 def babelkw(*args, **kw):
     return settings.SITE.babelkw(*args, **kw)
+babel_values = babelkw  # old alias for backwards compatibility
 
 
 def babelattr(*args, **kw):
     return settings.SITE.babelattr(*args, **kw)
-babel_values = babelkw  # old alias for backwards compatibility
 
 
 class UnresolvedModel(object):
@@ -490,10 +491,11 @@ def resolve_field(name, app_label=None):
     # return UnresolvedField(name)
 
 
-def navinfo(qs, elem):
+def navinfo(qs, elem, limit=None):
     """Return a dict with navigation information for the given model
-    instance `elem` within the given queryset.  The dictionary
-    contains the following keys:
+    instance `elem` within the given queryset.
+
+    The returned dictionary contains the following keys:
 
     :recno:   row number (index +1) of elem in qs
     :first:   pk of the first element in qs (None if qs is empty)
@@ -501,31 +503,37 @@ def navinfo(qs, elem):
     :next:    pk of the next element in qs (None if qs is empty)
     :last:    pk of the last element in qs (None if qs is empty)
     :message: text "Row x of y" or "No navigation"
+    :id_list: list of the primary keys
+
+    Used by :meth:`lino.core.actors.Actor.get_navinfo`.
 
     """
     first = None
     prev = None
     next = None
+    prev_page = None
+    next_page = None
     last = None
     recno = 0
     message = None
+    page_num = None
+    offset = None
+
     #~ LEN = ar.get_total_count()
     if isinstance(qs, (list, tuple)):
-        LEN = len(qs)
+        num = len(qs)
         id_list = [obj.pk for obj in qs]
         #~ logger.info('20130714')
     else:
-        LEN = qs.count()
+        num = qs.count()
         # this algorithm is clearly quicker on queries with a few thousand rows
         id_list = list(qs.values_list('pk', flat=True))
-    if LEN > 0:
-        """
-        Uncommented the following assert because it failed in certain circumstances
-        (see `/blog/2011/1220`)
-        """
-        #~ assert len(id_list) == ar.total_count, \
-            #~ "len(id_list) is %d while ar.total_count is %d" % (len(id_list),ar.total_count)
-        #~ print 20111220, id_list
+    if num > 0:
+        # Uncommented the following assert because it failed in certain circumstances
+        # (see `/blog/2011/1220`)
+        # assert len(id_list) == ar.total_count, \
+            # "len(id_list) is %d while ar.total_count is %d" % (len(id_list),ar.total_count)
+        # print 20111220, id_list
         try:
             i = id_list.index(elem.pk)
         except ValueError:
@@ -534,17 +542,25 @@ def navinfo(qs, elem):
             recno = i + 1
             first = id_list[0]
             last = id_list[-1]
+            if limit:
+                page_num = i // limit
+                offset = limit * page_num
             if i > 0:
-                prev = id_list[i - 1]
+                prev = id_list[i-1]
+                if limit:
+                    prev_page = id_list[max(0, offset-limit+1)]
             if i < len(id_list) - 1:
                 next = id_list[i + 1]
+                if limit:
+                    next_page = id_list[min(num-1, offset+limit+2)]
             message = _("Row %(rowid)d of %(rowcount)d") % dict(
-                rowid=recno, rowcount=LEN)
+                rowid=recno, rowcount=num)
     if message is None:
         message = _("No navigation")
     return dict(
         first=first, prev=prev, next=next, last=last, recno=recno,
-        message=message)
+        message=message, prev_page=prev_page, next_page=next_page,
+        offset=offset)
 
 
 # class Handle(object):
@@ -567,10 +583,12 @@ def navinfo(qs, elem):
 
 class Parametrizable(object):
     """
-    Base class for both Actors and Actions.
+    Base class for both Actors and Actions.  See :doc:`/dev/parameters`.
 
-    This is a pseudo-mixins with common functionality for both actors
-    and actions,
+    This is a pseudo-mixins that groups the common functionality for both actors
+    and actions. It's not a real mixin because :class:`Actor` must override
+    every method as a class method (because actors are class objects while
+    actions are class instances).
 
     .. method:: FOO_choices
 
@@ -583,37 +601,11 @@ class Parametrizable(object):
     active_fields = None  # 20121006
     master_field = None
     known_values = None
-
     parameters = None
-    """
-    User-definable parameter fields for this actor or action.  Set this
-    to a `dict` of `name = models.XyzField()` pairs.
-
-    On an actor you can alternatively or additionally implement a
-    class method :meth:`lino.core.actors.Actor.setup_parameters`.
-
-    TODO: write documentation.
-    """
-
     params_layout = None
-    """
-    The layout to be used for the parameter panel.
-    If this table or action has parameters, specify here how they
-    should be laid out in the parameters panel.
-    """
-
     params_panel_hidden = True
-    """
-    If this table has parameters, set this to True if the parameters
-    panel should be initially hidden when this table is being
-    displayed.
-    """
-
+    params_panel_pos = "top"  # allowed values "top", "bottom", "left" and "right"
     use_detail_param_panel = False
-    """
-    Set to true if you want the params panel to be displayed in the detail view.
-    Used only in :class:`lino_xl.lib.cal.CalView`.
-    """
 
     _layout_class = NotImplementedError
 
@@ -627,10 +619,11 @@ class Parametrizable(object):
 
     def check_params(self, pv):
         """Called when a request comes in."""
-        # Actor overrides this as a class method
         if isinstance(self.parameters, ParameterPanel):
             self.parameters.check_values(pv)
 
+    def hide_editing(cls, user_type):
+        return False
 
 class ParameterPanel(object):
     """
@@ -690,6 +683,7 @@ class ParameterPanel(object):
         """
         pass
 
+
 class PseudoRequest(object):
     """A Django HTTP request which isn't really one.
 
