@@ -1,7 +1,7 @@
 from builtins import str
 # -*- coding: UTF-8 -*-
 # Copyright 2011 Rumma & Ko Ltd
-# License: BSD (see file COPYING for details)
+# License: GNU Affero General Public License v3 (see file COPYING for details)
 
 import logging
 logger = logging.getLogger(__name__)
@@ -13,8 +13,8 @@ from os.path import join
 
 from django.db import models
 from django.conf import settings
-from django.utils.translation import ugettext as _
-from django.utils.encoding import force_text
+from django.utils.translation import gettext as _
+from django.utils.encoding import force_str
 from django.core.management import call_command
 from django.core.management.base import BaseCommand, CommandError
 from django.db.models import loading
@@ -68,11 +68,11 @@ def model_overview(model):
 
     def verbose_name(f):
         settings.SITE.set_language(None)
-        label_en = force_text(_(f.verbose_name))
+        label_en = force_str(_(f.verbose_name))
         babel_labels = []
         for lng in settings.SITE.languages[1:]:
             dbutils.set_language(lng.django_code)
-            label = force_text(_(f.verbose_name))
+            label = force_str(_(f.verbose_name))
             if label != label_en:
                 babel_labels.append(label)
         if babel_labels:
@@ -87,7 +87,7 @@ def model_overview(model):
         ]
         #~ for lng in babel.AVAILABLE_LANGUAGES:
             #~ babel.set_language(lng)
-            #~ cells.append(force_text(_(f.verbose_name)))
+            #~ cells.append(force_str(_(f.verbose_name)))
         #~ cells.append(f.help_text)
         return cells
     rows = [rowfmt(f) for f in model._meta.fields]
@@ -107,7 +107,7 @@ def model_overview(model):
 def rptlist(l):
     return ', '.join([
         ":ref:`%s (%s) <%s>`" % (str(rpt),
-                                 force_text(rpt.label), report_ref(rpt))
+                                 force_str(rpt.label), report_ref(rpt))
         for rpt in l])
 
 
