@@ -1,6 +1,6 @@
 # -*- coding: UTF-8 -*-
 # Copyright 2009-2020 Rumma & Ko Ltd
-# License: BSD (see file COPYING for details)
+# License: GNU Affero General Public License v3 (see file COPYING for details)
 """Defines the model mixins :class:`Sequenced` and
 :class:`Hierarchical`.
 
@@ -15,7 +15,7 @@ order which can be manipulated by the user using actions
 # from builtins import object
 
 from django.db import models
-from django.utils.translation import ugettext_lazy as _
+from django.utils.translation import gettext_lazy as _
 from django.core.exceptions import ValidationError
 
 from lino.core import actions
@@ -385,6 +385,9 @@ class Hierarchical(Duplicable):
                 return True
             p = p.parent
 
+    def get_ancestor(self):
+        return self.get_parental_line()[0]
+
     def get_parents(self):
         rv = []
         p = self.parent
@@ -429,3 +432,78 @@ class Hierarchical(Duplicable):
         clan |= set(l3)
         # print 20150421, projects
         return clan
+
+    def whole_tree(self):
+        """
+        Returns a tuple of tuples.
+        The first item is the top most ancestor and the second item is a tuple
+        of all the children of the ancestor.
+        A children is wrappend inside another tuple where the first item is the
+        children and second item is None when the children has no grandchildren
+        otherwise the second item will be another tuple of grandchildren
+        and so on.
+        """
+        items = []
+        parent = self.get_ancestor()
+        items.append(parent)
+        child = parent.children.all()
+        def append_to_tree(parent, child, tree):
+            if child.count() == 0:
+                return
+            else:
+                t = tree
+                for c in child:
+                    if c in items:
+                        continue
+                    items.append(c)
+                    t = t + ((c, append_to_tree(c, c.children.all(), tree)),)
+                return t
+        return (parent, append_to_tree(parent, child, tuple()))
+
+    @fields.htmlbox()
+    def treeview_panel(self, ar):
+        if ar is None:
+            return None
+        openned = self.get_parental_line()
+        ancestor = openned[0]
+        tab = "»"
+
+        folder_closed = "🗀 " # U+1F5C0
+        folder_open = "🗁 " # U+1F5C1
+        folder_with_child_closed = "📁 " # U+1F4C1 "🗂 "
+        folder_with_child_open = "📂 " # U+1F4C2
+
+        et = E.div()
+
+        def append_to_et(child, i=0):
+            def get_text(child):
+                return (tab * i) + (
+                    folder_open if child in openned and child.children.count() == 0
+                    else folder_with_child_open if child in openned
+                    else folder_closed if child.children.count() == 0
+                    else folder_with_child_closed
+                    ) + str(child)
+            et.append(ar.goto_pk(child.id, get_text(child)))
+            if child.children.count() > 0:
+                i += 1
+                for c in child.children.all():
+                    if c in openned:
+                        et.append(E.br())
+                        append_to_et(c, i)
+                    else:
+                        et.append(E.br())
+                        et.append(ar.goto_pk(c.id, get_text(c)))
+                i -= 1
+            if child == self:
+                sar = ar.actor.request_from(ar, master_instance=self, is_on_main_actor=False)
+                # sar = ar.spawn_request(master_instance=self, is_on_main_actor=False)
+                if ar.actor.insert_action is not None:
+                    ir = ar.actor.insert_action.request_from(sar)
+                    if ir.get_permission():
+                        btn = ir.ar2button()
+                        if len(et):
+                            et.append(E.br())
+                        et.append(btn)
+
+        append_to_et(ancestor)
+        return ar.html_text(et)
