@@ -1,6 +1,6 @@
 # -*- coding: UTF-8 -*-
 # Copyright 2011-2014 Rumma & Ko Ltd
-# License: BSD (see file COPYING for details)
+# License: GNU Affero General Public License v3 (see file COPYING for details)
 
 """The :xfile:`models.py` module for the
 :mod:`lino.modlib.awesomeuploader` app.
@@ -13,7 +13,7 @@ import logging
 logger = logging.getLogger(__name__)
 
 from django.conf import settings
-from django.utils.translation import ugettext_lazy as _
+from django.utils.translation import gettext_lazy as _
 from lino.api import dd, rt
 
 
