@@ -1,6 +1,6 @@
 # -*- coding: UTF-8 -*-
-# Copyright 2009-2020 Rumma & Ko Ltd
-# License: BSD (see file COPYING for details)
+# Copyright 2009-2021 Rumma & Ko Ltd
+# License: GNU Affero General Public License v3 (see file COPYING for details)
 
 """
 Defines the :class:`ExtRenderer` class.
@@ -14,7 +14,7 @@ import time
 from django.conf import settings
 from django.db import models
 from django.utils import translation
-from django.utils.translation import ugettext as _
+from django.utils.translation import gettext as _
 
 from etgen.html import E, iselement
 
@@ -283,6 +283,8 @@ class ExtRenderer(JsRenderer, JsCacheRenderer):
                 rp = None
             else:
                 rp = ar.requesting_panel
+                # if ar.master_instance is not None:
+                #     raise Exception("20210424")
             if status:
                 return "Lino.%s.run(%s,%s)" % (
                     bound_action.full_name(),
@@ -386,7 +388,7 @@ class ExtRenderer(JsRenderer, JsCacheRenderer):
             return '<div class="htmlText">{0}</div>'.format(html)
         if not iselement(html):
             raise Exception("{!r} is not an element".format(html))
-        if html.tag in ('div', 'span'):
+        if html.tag in ('div', 'span', 'p'):
             html.set('class', 'htmlText')
             return html
         return E.div(html, **{'class': 'htmlText'})
@@ -461,80 +463,81 @@ class ExtRenderer(JsRenderer, JsCacheRenderer):
     def html_page_user(self, request, site):
         """Build the "user menu", i.e. the menu in the top right corner.
         """
-        if settings.SITE.user_model is not None:
-            if request.user.is_authenticated:
-                if request.subst_user:
-                    yield "Lino.set_subst_user(%s,%s);" % (
-                        py2js(request.subst_user.id),
-                        py2js(str(request.subst_user)))
-                    user_text = str(request.user) + \
-                        " (" + _("as") + " " + \
-                        str(request.subst_user) + ")"
-                else:
-                    yield "Lino.set_subst_user();"
-                    user_text = str(request.user)
-
-                user = request.user
-
-                yield "Lino.user = %s;" % py2js(
-                    dict(id=user.id, name=str(user)))
-
-                a = rt.models.users.MySettings.default_action
-                handler = self.action_call(None, a, dict(record_id=user.pk))
-                handler = "function(){%s}" % handler
-                mysettings = dict(text=_("My settings"),
-                                  handler=js_code(handler))
-                login_menu_items = [mysettings]
-
-                authorities = user.get_authorities()
-                if len(authorities):
-                    act_as = [
-                        dict(text=t, handler=js_code(
-                            "function(){Lino.set_subst_user(%s, %s)}" % (v, py2js(t))))
-                        for v, t in authorities]
-                            #~ for v,t in user.get_received_mandates()]
-                    act_as.insert(0, dict(
-                        text=_("Myself"),
-                        handler=js_code("function(){Lino.set_subst_user(null)}")))
-                    act_as = dict(text=_("Act as..."), menu=dict(items=act_as))
-
-                    login_menu_items.insert(0, act_as)
-
-                if site.remote_user_header is None:
-                    # 20170921
-                    a = rt.models.users.MySettings.get_action_by_name('sign_out')
-                    # a = user.sign_out.bound_action
-                    js = self.action_call(None, a, {})
-                    js = "function(){%s}" % js
-                    login_menu_items.append(
-                        dict(text=a.get_button_label(), handler=js_code(js)))
-                # the following was never used
-                #     if auth.get_auth_middleware().can_change_password(request, request.user):
-                #         login_menu_items.append(
-                #             dict(text=_("Change password"), handler=js_code('Lino.change_password')))
-                #         login_menu_items.append(
-                #             dict(text=_("Forgot password"), handler=js_code('Lino.forgot_password')))
-
-                login_menu = dict(
-                    text=user_text,
-                    menu=dict(items=login_menu_items))
-
-                yield "Lino.main_menu = Lino.main_menu.concat(['->',%s]);" % py2js(login_menu)
-
+        if settings.SITE.user_model is None:
+            return
+        if request.user.is_authenticated:
+            if request.subst_user:
+                yield "Lino.set_subst_user(%s,%s);" % (
+                    py2js(request.subst_user.id),
+                    py2js(str(request.subst_user)))
+                user_text = str(request.user) + \
+                    " (" + _("as") + " " + \
+                    str(request.subst_user) + ")"
             else:
-                ba = rt.models.users.UsersOverview.get_action_by_name('sign_in')
-                js = self.action_call(None, ba, {})
+                yield "Lino.set_subst_user();"
+                user_text = str(request.user)
+
+            user = request.user
+
+            yield "Lino.user = %s;" % py2js(
+                dict(id=user.id, name=str(user)))
+
+            a = rt.models.users.MySettings.default_action
+            handler = self.action_call(None, a, dict(record_id=user.pk))
+            handler = "function(){%s}" % handler
+            mysettings = dict(text=_("My settings"),
+                              handler=js_code(handler))
+            login_menu_items = [mysettings]
+
+            authorities = user.get_authorities()
+            if len(authorities):
+                act_as = [
+                    dict(text=t, handler=js_code(
+                        "function(){Lino.set_subst_user(%s, %s)}" % (v, py2js(t))))
+                    for v, t in authorities]
+                        #~ for v,t in user.get_received_mandates()]
+                act_as.insert(0, dict(
+                    text=_("Myself"),
+                    handler=js_code("function(){Lino.set_subst_user(null)}")))
+                act_as = dict(text=_("Act as..."), menu=dict(items=act_as))
+
+                login_menu_items.insert(0, act_as)
+
+            if site.remote_user_header is None:
+                # 20170921
+                a = rt.models.users.MySettings.get_action_by_name('sign_out')
+                # a = user.sign_out.bound_action
+                js = self.action_call(None, a, {})
                 js = "function(){%s}" % js
-                login_buttons = [ dict(
-                    xtype="button", text=ba.get_button_label(),
-                    handler=js_code(js)) ]
-                # login_buttons = [
-                #     dict(xtype="button", text=_("Log in"),
-                #          handler=js_code('Lino.show_login_window')),
-                #     #~ dict(xtype="button",text="Register",handler=Lino.register),
-                # ]
-                yield "Lino.main_menu = \
-                Lino.main_menu.concat(['->', %s]);" % py2js(login_buttons)
+                login_menu_items.append(
+                    dict(text=a.get_button_label(), handler=js_code(js)))
+            # the following was never used
+            #     if auth.get_auth_middleware().can_change_password(request, request.user):
+            #         login_menu_items.append(
+            #             dict(text=_("Change password"), handler=js_code('Lino.change_password')))
+            #         login_menu_items.append(
+            #             dict(text=_("Forgot password"), handler=js_code('Lino.forgot_password')))
+
+            login_menu = dict(
+                text=user_text,
+                menu=dict(items=login_menu_items))
+
+            yield "Lino.main_menu = Lino.main_menu.concat(['->',%s]);" % py2js(login_menu)
+
+        else:
+            ba = rt.models.users.UsersOverview.get_action_by_name('sign_in')
+            js = self.action_call(None, ba, {})
+            js = "function(){%s}" % js
+            login_buttons = [ dict(
+                xtype="button", text=ba.get_button_label(),
+                handler=js_code(js)) ]
+            # login_buttons = [
+            #     dict(xtype="button", text=_("Log in"),
+            #          handler=js_code('Lino.show_login_window')),
+            #     #~ dict(xtype="button",text="Register",handler=Lino.register),
+            # ]
+            yield "Lino.main_menu = \
+            Lino.main_menu.concat(['->', %s]);" % py2js(login_buttons)
 
     def before_row_edit(self, panel):
         from lino.core.actions import get_view_permission
@@ -579,6 +582,15 @@ class ExtRenderer(JsRenderer, JsCacheRenderer):
         menu = settings.SITE.get_site_menu(user_type)
         menu.add_item(
             'home', _("Home"), javascript="Lino.handle_home_button()")
+
+        if len(settings.SITE.languages) > 1:
+            for lang in settings.SITE.languages:
+                # if lang.django_code == requested_language:
+                menu.add_item(
+                    lang.django_code, lang.name,
+                    javascript='Lino.set_user_language("{}")'.format(
+                        lang.django_code))
+
         f.write("Lino.main_menu = %s;\n" % py2js(menu))
 
         """Call Ext.namespace for *all* actors because
@@ -616,13 +628,27 @@ class ExtRenderer(JsRenderer, JsCacheRenderer):
             Return True if the given form layout `fl` is needed for
             user_type.
             """
+            # print("20210223", lh.layout._datasource)
+            # if str(lh.layout._datasource) == "courses.CourseDetail":
+            # if str(lh.layout._datasource) == "courses.Activities":
+            #     print("20210223 must_render",
+            #         user_type, lh.main.get_view_permission(user_type),
+            #         lh.layout)
+
             if not lh.main.get_view_permission(user_type):
                 return False
-            if lh.layout._datasource.get_view_permission(user_type):
-                return True
-            for ds in lh.layout._other_datasources:
+            for ds in lh.layout.get_datasources():
                 if ds.get_view_permission(user_type):
                     return True
+            # if lh.layout._datasource.get_view_permission(user_type):
+            #     # if str(lh.layout._datasource) == "courses.Activities":
+            #     #     print("20210223 must_render returns True", lh.layout._datasource)
+            #     return True
+            # for ds in lh.layout._other_datasources:
+            #     if ds.get_view_permission(user_type):
+            #         # if str(lh.layout._datasource) == "courses.Activities":
+            #         #     print("20210223 must_render returns True", ds)
+            #         return True
             return False
 
         #~ f.write('\n/* Application FormPanel subclasses */\n')
@@ -696,6 +722,8 @@ class ExtRenderer(JsRenderer, JsCacheRenderer):
                 elif ba.action.action_name:
                     is_custom = False
                     for parent in window_actions:
+                        if parent.window_type is None:
+                            raise Exception("20210514 {}.{}".format(rpt, parent))
                         if ba.action.is_callable_from(parent):
                             is_custom = True
                             break
@@ -856,7 +884,7 @@ class ExtRenderer(JsRenderer, JsCacheRenderer):
         yield ""
         yield "Lino.%s = Ext.extend(Ext.form.FormPanel, {" % \
             dh.layout._formpanel_name
-        for k, v in list(dh.main.ext_options().items()):
+        for k, v in dh.main.ext_options().items():
             #~ if k != 'items':
             if not k in self.SUPPRESSED:
                 yield "  %s: %s," % (k, py2js(v))
@@ -865,7 +893,20 @@ class ExtRenderer(JsRenderer, JsCacheRenderer):
             yield "  layout: 'hbox',"
         else:
             yield "  layout: 'form',"
-        yield "  autoHeight: true,"
+        # 20210501
+        if dh.layout._datasource.params_panel_pos == "top":
+            region = "north"
+            yield "  autoHeight: true,"
+        elif dh.layout._datasource.params_panel_pos == "bottom":
+            region = "south"
+            yield "  autoHeight: true,"
+        elif dh.layout._datasource.params_panel_pos == "left":
+            region = "west"
+        elif dh.layout._datasource.params_panel_pos == "right":
+            region = "east"
+        else:
+            region = "north"
+        yield "  region: '%s'," % region
         #~ if dh.layout.window_size and dh.layout.window_size[1] == 'auto':
             #~ yield "  autoHeight: true,"
         yield "  initComponent : function() {"
@@ -883,13 +924,18 @@ class ExtRenderer(JsRenderer, JsCacheRenderer):
             msg += ", main elements: %r" % dh.main.elements
             # raise Exception(msg)
             print((20150717, msg))
+        fields = [e for e in dh.main.walk()
+            if isinstance(e, ext_elems.FieldElement)]
         yield "    this.items = %s;" % py2js(dh.main.elements)
-        yield "    this.fields = %s;" % py2js(
-            [e for e in dh.main.walk()
-             if isinstance(e, ext_elems.FieldElement)])
+        yield "    this.fields = %s;" % py2js(fields)
         yield "    Lino.%s.superclass.initComponent.call(this);" % \
             dh.layout._formpanel_name
         yield "  }"
+        if region in ("west", "east"):
+            width = 20
+            for f in fields:
+                width = max(width, f.width or f.preferred_width)
+            yield "  ,width: Lino.chars2width(%s)" % width
         yield "});"
         yield ""
 
@@ -971,7 +1017,7 @@ class ExtRenderer(JsRenderer, JsCacheRenderer):
             yield "  autoHeight: true,"
         if settings.SITE.is_installed('contenttypes') and issubclass(tbl, dbtables.Table):
             yield "  content_type: %s," % py2js(ContentType.objects.get_for_model(tbl.model).pk)
-        if not tbl.editable:
+        if tbl.hide_editing(get_user_profile()):
             yield "  disable_editing: true,"
         if not tbl.auto_apply_params:
             yield "  auto_apply_params: false,"
@@ -1034,7 +1080,7 @@ class ExtRenderer(JsRenderer, JsCacheRenderer):
         yield "Lino.%sPanel = Ext.extend(Lino.%s,{" % (
             action.full_name(), dtl._formpanel_name)
         yield "  empty_title: %s," % py2js(action.get_button_label())
-        if action.action.hide_navigator:
+        if rpt.hide_navigator or action.action.hide_navigator:
             yield "  hide_navigator: true,"
 
         if rh.actor.params_panel_hidden:
@@ -1044,7 +1090,7 @@ class ExtRenderer(JsRenderer, JsCacheRenderer):
             yield "  save_action_name: %s," % py2js(
                 action.action.save_action_name)
         yield "  ls_bbar_actions: %s," % py2js(
-            self.toolbar(rpt.get_toolbar_actions(action.action)))
+            self.toolbar(rpt.get_toolbar_actions(action.action, get_user_profile())))
         yield "  ls_url: %s," % py2js(rpt.actor_url())
         if action.action != rpt.default_action.action:
             yield "  action_name: %s," % py2js(action.action.action_name)
@@ -1094,15 +1140,15 @@ class ExtRenderer(JsRenderer, JsCacheRenderer):
         kw.update(cell_edit=rh.actor.cell_edit)
         kw.update(focus_on_quick_search=rh.actor.focus_on_quick_search)
         kw.update(ls_bbar_actions=self.toolbar(
-            rh.actor.get_toolbar_actions(rh.actor.default_action.action)))
-        kw.update(ls_grid_configs=[gc.data for gc in rh.actor.grid_configs])
-        kw.update(gc_name=constants.DEFAULT_GC_NAME)
+            rh.actor.get_toolbar_actions(rh.actor.default_action.action, get_user_profile())))
+        # kw.update(ls_grid_configs=[gc.data for gc in rh.actor.grid_configs])
+        # kw.update(gc_name=constants.DEFAULT_GC_NAME)
         #~ if action != rh.actor.default_action:
             #~ kw.update(action_name=action.name)
         #~ kw.update(content_type=rh.report.content_type)
 
         vc = dict(emptyText=_("No data to display."))
-        if rh.actor.editable:
+        if rh.actor.hide_editing(get_user_profile()):
             vc.update(getRowClass=js_code('Lino.getRowClass'))
         if rh.actor.auto_fit_column_widths:
             vc.update(forceFit=True)
@@ -1118,19 +1164,19 @@ class ExtRenderer(JsRenderer, JsCacheRenderer):
 
         kw.update(viewConfig=vc)
 
-        if not rh.actor.editable:
+        if not rh.actor.hide_editing(get_user_profile()):
             kw.update(disable_editing=True)
         if rh.actor.params_panel_hidden:
             kw.update(params_panel_hidden=True)
 
-        if rh.actor.start_at_bottom:
-            kw.update(start_at_bottom=True)
+        # if rh.actor.start_at_bottom:
+        #     kw.update(start_at_bottom=True)
         kw.update(page_length=rh.actor.page_length)
         kw.update(stripeRows=True)
 
         #~ if rh.actor.master:
         kw.update(title=rh.actor.label)
-        if rh.actor.editable:
+        if not rh.actor.hide_editing(get_user_profile()):
             kw.update(
                 disabled_fields_index=rh.store.column_index('disabled_fields'))
 
@@ -1297,3 +1343,45 @@ class ExtRenderer(JsRenderer, JsCacheRenderer):
             yield "GEN_TIMESTAMP = %s;" % py2js(rt.settings.SITE.kernel.code_mtime)
 
         return '\n'.join(fn())
+
+    def goto_instance(self, ar, obj, detail_action=None, **kw):
+        """
+
+        See JsRenderer.goto_instance(), but when this is called while the detail
+        window is already open (only on another record), then we don't want to
+        redirect to another page because that would take more time.
+
+        This is a utility wrapper around :meth:`set_response` which sets
+        either `data_record` or a `record_id`.
+
+        Usually `data_record`, except if it is a `file upload
+        <https://docs.djangoproject.com/en/3.1/topics/http/file-uploads/>`_
+        where some mysterious decoding problems (:blogref:`20120209`)
+        force us to return a `record_id` which has the same visible
+        result but using an additional GET.
+
+        If the calling window is a detail on the same table, then it
+        should simply get updated to the given record. Otherwise open a
+        new detail window.
+
+        If the detail layout of the current actor can be used for the
+        object to be displayed, we don't want to open a new detail
+        window.
+
+
+        """
+        if ar.actor is not None:
+            da = detail_action or obj.get_detail_action(ar)
+            if da is None:
+                # print("20210325 goto_instance no da")
+                return
+            # print("20201230b goto_instance", da.actor, ar.actor)
+            if da.actor == ar.actor:
+                ar.set_response(detail_handler_name=da.full_name())
+                if ar.actor.handle_uploaded_files is not None:
+                    ar.set_response(record_id=obj.pk)
+                else:
+                    ar.set_response(
+                        data_record=ar.elem2rec_detailed(obj))
+                return
+        return super(ExtRenderer, self).goto_instance(ar, obj, detail_action, **kw)
