@@ -1,5 +1,5 @@
 # Copyright 2015-2020 Rumma & Ko Ltd
-# License: BSD (see file COPYING for details)
+# License: GNU Affero General Public License v3 (see file COPYING for details)
 
 """
 This initializes the `SITE.jinja_env` object.  Compare with
@@ -13,7 +13,7 @@ import jinja2
 
 from django.conf import settings
 
-from django.utils.translation import ugettext
+from django.utils.translation import gettext
 from django.utils.translation import pgettext
 
 from etgen import html as xghtml
@@ -57,7 +57,8 @@ class JinjaRenderer(HtmlRenderer):
             #~ logger.info("20130717 jinja loader %s %s",name,SUBDIR_NAME)
             if isdir(join(dirname(m.__file__), SUBDIR_NAME)):
                 loader = jinja2.PackageLoader(name, SUBDIR_NAME)
-                loaders.append(loader)
+                loaders.insert(0, loader)
+                # loaders.append(loader)
                 prefix_loaders[name] = loader
         settings.SITE.for_each_app(func)
 
@@ -113,6 +114,7 @@ class JinjaRenderer(HtmlRenderer):
             len=len,
             E=E,
             tostring=tostring,
+            isinstance=isinstance,
             ar=BaseRequest(renderer=settings.SITE.kernel.default_renderer),
             # _=_,
             now=datetime.datetime.now(),
@@ -137,7 +139,7 @@ class JinjaRenderer(HtmlRenderer):
         )
 
         def translate(s):
-            return ugettext(str(s))
+            return gettext(str(s))
         self.jinja_env.globals.update(_=translate)
 
         def ptranslate(ctx, s):
