@@ -1,5 +1,5 @@
-# Copyright 2008-2019 Rumma & Ko Ltd
-# License: BSD (see file COPYING for details)
+# Copyright 2008-2021 Rumma & Ko Ltd
+# License: GNU Affero General Public License v3 (see file COPYING for details)
 
 """Adds functionality for managing notification messages.
 
@@ -44,6 +44,14 @@ class Plugin(ad.Plugin):
 
     """
 
+    mark_seen_when_sent = False
+    """
+
+    When this is True, Lino marks notification messages as `seen` when they have
+    been sent via email.
+
+    """
+
     media_name = 'js'
 
     # email_subject_template = "Message about {obj.owner}"
