@@ -1,6 +1,6 @@
 # -*- coding: UTF-8 -*-
 # Copyright 2011-2020 Rumma & Ko Ltd
-# License: BSD (see file COPYING for details)
+# License: GNU Affero General Public License v3 (see file COPYING for details)
 
 import logging ; logger = logging.getLogger(__name__)
 import json
@@ -221,6 +221,8 @@ class Message(UserAuthored, Controllable, Created):
                 rt.send_email(subject, sender, body, [user.email])
                 for msg in messages:
                     msg.sent = timezone.now()
+                    if dd.plugins.notify.mark_seen_when_sent:
+                        msg.seen = timezone.now()
                     msg.save()
 
     def send_browser_message_for_all_users(self, user):
@@ -293,7 +295,7 @@ class Messages(dd.Table):
     # cell_edit = False
 
     card_layout = """
-    user seen 
+    user seen
     subject
     workflow_buttons
     """
