@@ -1,12 +1,12 @@
 # -*- coding: UTF-8 -*-
-# Copyright 2009-2019 Rumma & Ko Ltd
-# License: BSD (see file COPYING for details)
+# Copyright 2009-2021 Rumma & Ko Ltd
+# License: GNU Affero General Public License v3 (see file COPYING for details)
 
 from django.conf import settings
-from django.utils.encoding import force_text
+from django.utils.encoding import force_str
 
 from django.db import models
-from django.utils.translation import ugettext_lazy as _
+from django.utils.translation import gettext_lazy as _
 
 from django.apps import apps ; get_models = apps.get_models
 
@@ -66,12 +66,12 @@ class SiteConfig(dd.Model):
     site_company = dd.ForeignKey(
         "contacts.Company",
         blank=True, null=True,
-        verbose_name=_("Site owner"),
+        verbose_name=_("Site operator"),
         related_name='site_company_sites')
 
 
     def __str__(self):
-        return force_text(_("Site Parameters"))
+        return force_str(_("Site Parameters"))
 
     def update(self, **kw):
         """
@@ -112,20 +112,18 @@ class SiteConfigs(dd.Table):
 
     model = 'system.SiteConfig'
     required_roles = dd.login_required(SiteStaff)
-    # default_action = actions.ShowDetail()
-    #~ has_navigator = False
+    hide_navigator = True
+    allow_delete = False
     hide_top_toolbar = True
-    #~ can_delete = perms.never
-    detail_layout = """
+
+    detail_layout = dd.DetailLayout("""
     default_build_method
     # lino.ModelsBySite
-    """
+    """, window_size=(60, 'auto'))
 
     @classmethod
     def get_default_action(cls):
-        return actions.ShowDetail(cls.detail_layout)
-
-
+        return cls.detail_action
 
     do_build = BuildSiteCache()
 
