@@ -1,94 +1,146 @@
 # -*- coding: UTF-8 -*-
-# Copyright 2011-2020 Rumma & Ko Ltd
-# License: BSD (see file COPYING for details)
+# Copyright 2011-2021 Rumma & Ko Ltd
+# License: GNU Affero General Public License v3 (see file COPYING for details)
+
+from etgen.html import E
 
 from django.db import models
 from django.conf import settings
+from django.utils import timezone
 
 from lino.api import dd, rt, _
 from lino.core.roles import SiteAdmin
 from lino.core import auth, layouts
+from lino.core.actions import SubmitInsert
+
+
+class CheckedSubmitInsert(SubmitInsert):
+    """Like the standard :class:`lino.core.actions.SubmitInsert`, but
+    checks certain things before accepting the new user.
+
+    """
+    def run_from_ui(self, ar, **kw):
+        obj = ar.create_instance_from_request()
+        qs = obj.__class__.objects.filter(username=obj.username)
+        if len(qs) > 0:
+            msg = _("The username {} is taken. "
+                    "Please choose another one").format(obj.username)
+
+            ar.error(msg)
+            return
+
+        def ok(ar2):
+            SubmitInsert.run_from_ui(self, ar, **kw)
+            # self.save_new_instance(ar2, obj)
+            ar2.success(_("Your request has been registered. "
+                          "An email will shortly be sent to {0}"
+                          "Please check your emails.").format(
+                              obj.email))
+            # ar2.set_response(close_window=True)
+            # logger.info("20140512 CheckedSubmitInsert")
+
+        ok(ar)
+
+
+class VerifyUser(dd.Action):
+    """Enter your verification code."""
+    label = _("Verify")
+    # http_method = 'POST'
+    # select_rows = False
+    # default_format = 'json'
+    required_roles = set([])
+    # required_roles = dd.login_required(SiteAdmin)
+    show_in_bbar = False
+    show_in_workflow = True
+    parameters = dict(
+        email=models.EmailField(_('e-mail address')),
+        verification_code=models.CharField(
+            _("Verification code"), max_length=50))
+    params_layout = """
+    email
+    # instruct
+    verification_code
+    """
+    # def get_action_title(self, ar):
+    #     return _("Register new user")
+
+    # @dd.constant()
+    # def instruct(self, *args):
+    #     return _("Enter the verification code you received.")
+
+    def get_action_permission(self, ar, obj, state):
+        if obj.is_verified():
+            return False
+        return super(
+            VerifyUser, self).get_action_permission(ar, obj, state)
+
+    def run_from_ui(self, ar, **kw):
+        assert len(ar.selected_rows) == 1
+        user = ar.selected_rows[0]
+        pv = ar.action_param_values
+        # qs = rt.models.users.User.objects.exclude(verification_code='')
+        # try:
+        #     user = qs.get(email=pv.email)
+        # except Exception:
+        #     msg = _("Invalid email address")
+        #     return ar.error(msg)
+        if user.verification_code != pv.verification_code:
+            msg = _("Invalid verification code")
+            return ar.error(msg)
+        user.verification_code = ''
+        user.save()
+        ar.success(_("User {} is now verified.").format(user))
+
 
 class SendWelcomeMail(dd.Action):
     label = _("Welcome mail")
-    if False:  # #1336
+    if True:  # #1336
         show_in_bbar = True
         show_in_workflow = False
     else:
         show_in_bbar = False
         show_in_workflow = True
-    button_text = u"\u2709"  # ✉
-    required_roles = dd.login_required(SiteAdmin)
-    parameters = dict(
-        email=models.EmailField(_('e-mail address')),
-        subject=models.CharField(_('Subject'), max_length=250),
-        #body_text=dd.RichTextField(_('Body text'), format='html'),
-    )
-    # params_layout = dd.Panel("""email
-    # subject
-    # body_text""", window_size=(60, 15))
+    button_text = "\u2709"  # ✉
+    # required_roles = dd.login_required(SiteAdmin)
 
-    # params_layout = dd.Panel("""
-    # email
-    # subject
-    # welcome_email_body""", window_size=(60, 15))
-
-    params_layout = """
-    email
-    subject
-    """
-
-    def action_param_defaults(self, ar, obj, **kw):
-        kw = super(SendWelcomeMail, self).action_param_defaults(
-            ar, obj, **kw)
-        if obj is not None:
-            kw.update(email=obj.email)
-        kw.update(subject=_("Welcome on {site}").format(
-            site=settings.SITE.title or settings.SITE.verbose_name))
-        # template = rt.get_template('users/welcome_email.eml')
-        # context = dict(obj=obj, E=E, rt=rt)
-        # body = template.render(**context)
-        # body = E.fromstring(body)
-        # kw.update(body_text=body)
-        return kw
-
-    # def get_action_permission(self, ar, obj, state):
-    #     user = ar.get_user()
-    #     if not obj.email:
-    #         return False
-    #     if not user.user_type.has_required_roles([SiteAdmin]):
-    #         return False
-    #     return super(
-    #         SendWelcomeMail, self).get_action_permission(ar, obj, state)
+    def get_action_permission(self, ar, obj, state):
+        if not obj.email:
+            return False
+        user = ar.get_user()
+        if user != self:
+            if not user.user_type.has_required_roles([SiteAdmin]):
+                return False
+        return super(
+            SendWelcomeMail, self).get_action_permission(ar, obj, state)
 
     def run_from_ui(self, ar, **kw):
-
-        sender = settings.SERVER_EMAIL
-        done_for = []
         assert len(ar.selected_rows) == 1
         obj = ar.selected_rows[0]
-        subject = ar.action_param_values.subject
-        email = ar.action_param_values.email
-        # body = ar.action_param_values.body_text
-        # body = "<body>" + body + "</body>"
-        email = "{} <{}>".format(obj, email)
+        sender = settings.SERVER_EMAIL
+        recipients = ["{} <{}>".format(obj, obj.email)]
 
-        body = obj.get_welcome_email_body(ar)
-        print(20170102, obj, email, body)
-        # send_welcome_email(obj)
-        rt.send_email(subject, sender, body, [email])
-        done_for.append(email)
+        def ok(ar):
+            subject = _("Welcome on {site}").format(
+                site=settings.SITE.title or settings.SITE.verbose_name)
 
-        msg = _("Welcome mail has been sent to {}.").format(
-            ', '.join(done_for))
-        ar.success(msg, alert=True)
+            body = obj.get_welcome_email_body(ar)
+            # print(20170102, obj, email, body)
+            # send_welcome_email(obj)
+            rt.send_email(subject, sender, body, recipients)
+
+            msg = _("Welcome mail has been sent to {}.").format(
+                ', '.join(recipients))
+            ar.success(msg, alert=True)
+
+        msg = _("Send welcome mail to {} ?").format(', '.join(recipients))
+        return ar.confirm(ok, msg)
 
 
 
 class ChangePassword(dd.Action):
     # button_text = u"\u205C"  # DOTTED CROSS (⁜)
     # button_text = u"\u2042"  # ASTERISM (⁂)
-    button_text = u"\u2731" # 'HEAVY ASTERISK' (✱)
+    button_text = "\u2731" # 'HEAVY ASTERISK' (✱)
     # icon_name = "disk"
     label = _("Change password")
 
@@ -151,7 +203,6 @@ class SignOut(dd.Action):
             _("User {} logged out.").format(user),
             goto_url=ar.renderer.front_end.build_plain_url())
 
-from etgen.html import E
 # from lino.core.fields import DisplayField, DummyField
 
 # class SocialAuthField(DisplayField):
@@ -166,6 +217,18 @@ from etgen.html import E
 #         return SocialAuthField()
 #     return DummyField()
 
+def validate_sessions_limit(request):
+    if dd.plugins.users.active_sessions_limit == -1:
+        return
+    qs = rt.models.sessions.Session.objects.filter(
+        expire_date__gt=timezone.now())
+    if request.session.session_key:
+        qs = qs.exclude(session_key=request.session.session_key)
+    if qs.count() >= dd.plugins.users.active_sessions_limit:
+        raise Warning(
+            _("There are more than {} active user sessions. Please try again later.").format(
+                dd.plugins.users.active_sessions_limit))
+
 
 class SignIn(dd.Action):
     label = _("Sign in")
@@ -183,6 +246,9 @@ class SignIn(dd.Action):
     http_method = "POST"
 
     def run_from_ui(self, ar, **kw):
+        # ipdict = dd.plugins.ipdict
+        # print("20210212 SignIn.run_from_ui()", ipdict.ip_records)
+        validate_sessions_limit(ar.request)
         pv = ar.action_param_values
         user = auth.authenticate(
             ar.request, username=pv.username, password=pv.password)
