@@ -1,6 +1,6 @@
 # -*- coding: UTF-8 -*-
 # Copyright 2011-2020 Rumma & Ko Ltd
-# License: BSD (see file COPYING for details)
+# License: GNU Affero General Public License v3 (see file COPYING for details)
 
 from django.db import models
 from django.conf import settings
@@ -17,15 +17,27 @@ from lino.mixins import DateRange
 
 from .choicelists import UserTypes
 from .mixins import UserAuthored  #, TimezoneHolder
-from .actions import ChangePassword, SignOut
-# from .actions import SendWelcomeMail
+from .actions import ChangePassword, SignOut, VerifyUser, CheckedSubmitInsert
+from .actions import SendWelcomeMail
 # from .actions import SignIn
 from lino.core.auth.utils import AnonymousUser
-from lino.modlib.about.choicelists import TimeZones
+from lino.modlib.about.choicelists import TimeZones, DateFormats
 from lino.modlib.printing.mixins import Printable
 
 from lino.core.roles import Supervisor
 
+import random
+import string
+
+def id_generator(size=6, chars=string.ascii_uppercase + string.digits):
+    # thanks to https://stackoverflow.com/questions/2257441/random-string-generation-with-upper-case-letters-and-digits-in-python
+    return ''.join(random.SystemRandom().choice(chars) for _ in range(size))
+
+
+partner_model = dd.plugins.users.partner_model
+
+SYNCHRONIZED_FIELDS = ('first_name', 'last_name', 'email')
+
 
 class UserManager(BaseUserManager):
     use_in_migrations = True
@@ -72,18 +84,31 @@ class User(AbstractBaseUser, Contactable, CreatedModified, Printable, DateRange)
     # seems that Django doesn't like nullable username
     # username = dd.NullCharField(_('Username'), max_length=30, unique=True)
     username = models.CharField(_('Username'), max_length=30, unique=True)
-
     user_type = UserTypes.field(blank=True)
     initials = models.CharField(_('Initials'), max_length=10, blank=True)
     first_name = models.CharField(_('First name'), max_length=30, blank=True)
     last_name = models.CharField(_('Last name'), max_length=30, blank=True)
     remarks = models.TextField(_("Remarks"), blank=True)  # ,null=True)
+    partner = dd.ForeignKey(partner_model,
+                blank=True, null=True,
+                related_name='users_by_partner')
+
+    if dd.plugins.users.use_verify:
+        verification_code = models.CharField(max_length=200, blank=True)
+        verify = VerifyUser()
+    else:
+        verify = None
+        verification_code = dd.DummyField()
 
     if settings.USE_TZ:
         time_zone = TimeZones.field(default='default')
     else:
         time_zone = dd.DummyField()
 
+    date_format = DateFormats.field(default='default')
+
+    submit_insert = CheckedSubmitInsert()
+    send_welcome_email = SendWelcomeMail()
     change_password = ChangePassword()
     # sign_in = SignIn()
     sign_out = SignOut()
@@ -99,6 +124,14 @@ class User(AbstractBaseUser, Contactable, CreatedModified, Printable, DateRange)
             return False
         return True
 
+    def is_verified(self):
+        if dd.plugins.users.use_verify:
+            return not self.verification_code
+        return False
+
+    def mark_verified(self):
+        self.verification_code = ''
+
     def get_as_user(self):
         """
         Overrides :meth:`lino_xl.lib.contacts.Partner.get_as_user`.
@@ -108,7 +141,7 @@ class User(AbstractBaseUser, Contactable, CreatedModified, Printable, DateRange)
     def get_full_name(self):
         if not self.first_name and not self.last_name:
             return self.initials or self.username or str(self.pk)
-        return u'{} {}'.format(self.first_name, self.last_name).strip()
+        return '{} {}'.format(self.first_name, self.last_name).strip()
 
     # name_column is now defined in core.model.Model
     # @dd.displayfield(_("Name"), max_length=15)
@@ -124,11 +157,17 @@ class User(AbstractBaseUser, Contactable, CreatedModified, Printable, DateRange)
     #     # elems.append(E.a("foo"))
     #     # return E.p(elems)
 
-    def get_person(self):
+    @dd.virtualfield(dd.ForeignKey('contacts.Person'))
+    def person(self, ar):
         if self.partner:
             return self.partner.get_mti_child('person')
 
-    person = property(get_person)
+    # person = property(get_person)
+
+    @dd.virtualfield(dd.ForeignKey('contacts.Company'))
+    def company(self, ar):
+        if self.partner:
+            return self.partner.get_mti_child('company')
 
     def is_editable_by_all(self):
         return False
@@ -155,37 +194,60 @@ class User(AbstractBaseUser, Contactable, CreatedModified, Printable, DateRange)
         rv = super(User, self).disabled_fields(ar)
         user = ar.get_user()
         if not user.user_type.has_required_roles([SiteAdmin]):
-            # LS 20191120 uncommented the following as it is useless anyway
-            # (there is no pluing named "online")
-            # if not dd.is_installed("online"):
-            #     rv.add("submit_insert")
-            #     rv.add("insert")
-            rv.add('send_email')
             rv.add('user_type')
+            rv.add('partner')
+            rv.add('send_email')
             if user != self:
                 rv.add('change_password')
         return rv
 
     def full_clean(self, *args, **kw):
-        p = self.get_person()
-        if p is not None and p != self:
-            for n in ('first_name', 'last_name', 'email', 'language'):
-                if not getattr(self, n):
-                    setattr(self, n, getattr(p, n))
-            #~ self.language = p.language
-        if not self.language:
-            #~ self.language = settings.SITE.DEFAULT_LANGUAGE.django_code
-            self.language = settings.SITE.get_default_language()
         if not self.password:
             self.set_unusable_password()
         # if not self.initials:
         #     if self.first_name and self.last_name:
         #         self.initials = self.first_name[0] + self.last_name[0]
         super(User, self).full_clean(*args, **kw)
+        self.sync_partner()
+        if not self.language:
+            self.language = settings.SITE.get_default_language()
+
+    def sync_partner(self, **kw):
+        if partner_model is None:
+            return
+        if self.user_type and isinstance(self.user_type.role, SiteAdmin):
+            # site admins are not automatically linked to a partner
+            return
+        p = self.partner
+        # if not issubclass(rt.models.contacts.Person, partner_model):
+        #     return
+        if p is None:
+            if self.is_verified():
+                for k in SYNCHRONIZED_FIELDS:
+                    v = getattr(self, k)
+                    if v:
+                        kw[k] = v
+                    else:
+                        return
+                p = rt.models.contacts.Person(**kw)
+                p.full_clean()
+                p.save()
+                self.partner = p
+        else:
+            p = p.get_mti_child('person')
+            if p is not None:
+                for k in SYNCHRONIZED_FIELDS:
+                    if not getattr(self, k):
+                        setattr(self, k, getattr(p, k))
 
     def on_create(self, ar):
+        self.email_changed(ar)
         self.start_date = dd.today()
-        super(User, self).on_create(ar)
+        return super(User, self).on_create(ar)
+
+    def email_changed(self, ar):
+        if self.email and dd.plugins.users.use_verify:
+            self.verification_code = id_generator(12)
 
     def get_received_mandates(self):
         #~ return [ [u.id,_("as %s")%u] for u in self.__class__.objects.all()]
@@ -263,7 +325,6 @@ class User(AbstractBaseUser, Contactable, CreatedModified, Printable, DateRange)
     # def do_send_email(self, ar):
     #     self.send_welcome_email()
 
-    # send_email = SendWelcomeMail()
     def usertext(self):
         return "{0} {1}, {3} ({2})".format(
             self.last_name, self.first_name, self.username, self.user_type)
@@ -307,22 +368,22 @@ class Authority(UserAuthored):
 
 dd.update_field(Authority, 'user', null=False)
 
-@dd.receiver(dd.pre_startup)
-def inject_partner_field(sender=None, **kwargs):
-
-    User = sender.models.users.User
-
-    if dd.is_installed('contacts'):
-        Partner = sender.models.contacts.Partner
-        if not issubclass(User, Partner):
-            dd.inject_field(User, 'partner', dd.ForeignKey(
-                'contacts.Partner', blank=True, null=True,
-                related_name='users_by_partner',
-                on_delete=models.PROTECT))
-            # a related_name is needed so that Avanti can have a Client
-            # who inherits from both Partner and UserAuthored
-            return
-    dd.inject_field(User, 'partner', dd.DummyField())
+# @dd.receiver(dd.pre_startup)
+# def inject_partner_field(sender=None, **kwargs):
+#
+#     User = sender.models.users.User
+#
+#     if dd.is_installed('contacts'):
+#         Partner = sender.models.contacts.Partner
+#         if not issubclass(User, Partner):
+#             dd.inject_field(User, 'partner', dd.ForeignKey(
+#                 'contacts.Partner', blank=True, null=True,
+#                 related_name='users_by_partner',
+#                 on_delete=models.PROTECT))
+#             # a related_name is needed so that Avanti can have a Client
+#             # who inherits from both Partner and UserAuthored
+#             return
+#     dd.inject_field(User, 'partner', dd.DummyField())
 
 
 class Permission(dd.Model):
