@@ -1,29 +1,19 @@
 # -*- coding: UTF-8 -*-
-# Copyright 2009-2020 Rumma & Ko Ltd
-# License: BSD (see file COPYING for details)
-
-# Note that this module may not have a docstring because any
-# global variable defined here will override the global
-# namespace of lino/__init__.py who includes it with execfile.
-
-# This module is part of the Lino test suite.
-# To test only this module:
-#
-#   $ python setup.py test -s tests.PackagesTests
+# Copyright 2009-2021 Rumma & Ko Ltd
+# License: GNU Affero General Public License v3 (see file COPYING for details)
+# python setup.py test -s tests.PackagesTests
 
 import sys
 
 SETUP_INFO = dict(
     name='lino',
-    version='21.1.0',
-    description="A framework for writing desktop-like web applications "
-                "using Django and ExtJS or React",
-    license='BSD-2-Clause',
-    # obsoletes=['djangosite', 'north'],
-    author='Luc Saffre',
-    author_email='luc@lino-framework.org',
-    url="http://www.lino-framework.org",
-    # ~ test_suite = 'lino.test_apps',
+    version='21.5.0',
+    description="A framework for writing desktop-like sustainably free "
+                "web applications using Django and ExtJS or React",
+    license='COPYING',
+    author='Rumma & Ko Ltd',
+    author_email='info@lino-framework.org',
+    url="https://gitlab.com/lino-framework/lino",
     test_suite='tests')
 
 install_requires = [
@@ -83,21 +73,16 @@ SETUP_INFO.update(install_requires=install_requires)
 
 SETUP_INFO.update(long_description="""
 
-Lino is a high-level framework for writing desktop-like customized database
-applications based on `Django <https://www.djangoproject.com/>`_ and `Sencha
-ExtJS <http://www.sencha.com/products/extjs/>`_. Lino applications are Django
-projects where the application developer does not need to write any URLconf,
-HTML nor CSS (`more <http://lino-framework.org/about/what.html>`__).
+This is the core package of the Lino framework.
 
-- This repository is considered an integral part of the Lino framework, which is
-  documented as a whole in the `Lino Book
-  <http://www.lino-framework.org/about/overview.html>`__.
+This repository is an integral part of the Lino framework, which is
+documented as a whole in the `Lino Book
+<https://www.lino-framework.org/about/overview.html>`__.
 
-- Your feedback is welcome.  Our `community page
-  <http://www.lino-framework.org/community>`__ explains how to contact us.
-
-- API changes to this repository are logged at
-  http://www.lino-framework.org/changes/
+- Code repository: https://gitlab.com/lino-framework/lino
+- Test results: https://gitlab.com/lino-framework/lino/-/pipelines
+- Feedback: https://community.lino-framework.org
+- Maintainer: https://www.saffre-rumma.net/
 
 """)
 
@@ -109,7 +94,7 @@ Environment :: Web Environment
 Framework :: Django
 Intended Audience :: Developers
 Intended Audience :: System Administrators
-License :: OSI Approved :: BSD License
+License :: OSI Approved :: GNU Affero General Public License v3
 Natural Language :: English
 Natural Language :: French
 Natural Language :: German
@@ -174,6 +159,7 @@ lino.modlib.system
 lino.modlib.tinymce
 lino.modlib.tinymce.fixtures
 lino.modlib.uploads
+lino.modlib.uploads.fixtures
 lino.modlib.users
 lino.modlib.users.fixtures
 lino.modlib.weasyprint
