@@ -1,6 +1,6 @@
 # -*- coding: UTF-8 -*-
 # Copyright 2009-2020 Rumma & Ko Ltd
-# License: BSD (see file COPYING for details)
+# License: GNU Affero General Public License v3 (see file COPYING for details)
 
 """:mod:`lino.utils` (the top-level module) contains a few often-used
 function for general use. It has also many subpackages and submodules.
@@ -30,7 +30,6 @@ function for general use. It has also many subpackages and submodules.
     jscompressor
     jsgen
     latex
-    log
     mdbtools
     media
     mldbc
@@ -190,7 +189,7 @@ def last_day_of_month(d):
     datetime.date(2016, 11, 30)
 
     Thanks to `stackoverflow.com
-    <http://stackoverflow.com/questions/42950/get-last-day-of-the-month-in-python>`_.
+    <https://stackoverflow.com/questions/42950/get-last-day-of-the-month-in-python>`_.
 
     """
     return d + relativedelta(day=31)
@@ -714,8 +713,8 @@ UNCAMEL_RE = re.compile('((?<=[a-z0-9])[A-Z]|(?!^)[A-Z](?=[a-z]))')
 def uncamel(s):
     """
 
-    Thanks to `nickl <http://stackoverflow.com/users/1522117/nickl>`_
-    in `Stackoverflow  <http://stackoverflow.com/questions/1175208>`_
+    Thanks to `nickl <https://stackoverflow.com/users/1522117/nickl>`_
+    in `Stackoverflow  <https://stackoverflow.com/questions/1175208>`_
 
     >>> from lino.utils import uncamel
     >>> uncamel('EventsByClient')
@@ -783,7 +782,7 @@ class SumCollector(object):
     renderer (:mod:`lino.modlib.jinja`) when rendering templates,
     which makes it a more complete solution for a problem asked also
     elsewhere, e.g. on `Stackoverflow
-    <http://stackoverflow.com/questions/7537439/how-to-increment-a-variable-on-a-for-loop-in-jinja-template>`__.
+    <https://stackoverflow.com/questions/7537439/how-to-increment-a-variable-on-a-for-loop-in-jinja-template>`__.
 
     """
     def __init__(self):
@@ -820,6 +819,14 @@ class SimpleSingleton(object):
         return cls._instance
 
 
+def get_client_ip_address(request):
+    # from https://stackoverflow.com/questions/4581789/how-do-i-get-user-ip-address-in-django
+    x_forwarded_for = request.META.get('HTTP_X_FORWARDED_FOR')
+    if x_forwarded_for:
+        ip = x_forwarded_for.split(',')[0]
+    else:
+        ip = request.META.get('REMOTE_ADDR')
+    return ip or "(unknown)"  # see ticket #2605
 
 
 def _test():
